/************************************************************************/
/*  Build_NC: Build NCSN .tbl files from the .csv file                  */
/************************************************************************/
/*
    Author:
	Stephane Zuzlewski
	UC Berkeley Seismological Laboratory
	stephane@seismo.berkeley.edu

    Purpose:

    Modification History:
    Date	Ver	Who	What
    ---------------------------------------------------------------------
    2002/03/14	1.0	SMZ	Initial coding.
    2002/08/01	1.1	SMZ	Now creates hardware .tbl files as well.
    2002/08/12	1.2	SMZ	Implements new changes discussed after
				meeting at USGS.
    2002/11/23	1.2	SMZ	Added 'NC.' prefix to serial numbers
    2003/02/18	1.2	SMZ	FIR Filter value of ~ is converted to NULL;
				Sensor Component has to be > 0.
    2003/02/20	1.2	SMZ	Added constraint (pin# < 640);
				Added start date in error line
    2003/02/02	1.2	SMZ	Duplicate channels are not written to .tbl
    				file anymore.
    2003/05/22	1.2	SMZ	Corrected bug with digitizer units.
    2003/06/25	1.2	SMZ	Datalogger serial numbers are now the
				concatenation of type and serial number.
				Cleaned up error messages based on Fred
				Klein feedback.
    2003/06/30	1.2	SMZ	Digitizer units are now counts/volts.
    2003/11/05	1.2	SMZ	Fred added new field to spreadsheet:
				"USGS_Decimation_Channel".
    2003/11/26	1.2	SMZ	Sensor & preamp serial numbers are now the
				concatenation of type and serial number.
    2004/02/17	1.2	SMZ	Networks and stations to ignore can be
				specified in config files.
				Stations to update can be specified via
				an option on the command line.
    2004/05/05	1.2	SMZ	Added special for case for dataloggers
				that have modules starting at 0 instead
				of 1.
    2004/05/13	1.2	SMZ	Channels with sample rate = 0 are non
				operational.
    2004/07/19	1.2	SMZ	Ignore channels for which the sensor or
    				preamp class file does not exist.
    2004/11/02	1.2	SMZ	Reads secondary datalogger information
				if primary one is missing.
    2006/09/05	1.3	SMZ	Now supports multiple networks.
    2008/02/20	1.4	SMZ	Fixed bug with preamp gains.
									*/
/************************************************************************/

#ifndef lint
static char sccsid[] = "%W% %G% %U%";
#endif

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <unistd.h>
#include <qlib2.h>

#define		VERSION		"1.4"
#define		info		stdout
#define		VERSION_FILE	"Version"
#define		CALDATE         "/usr/local/bin/caldate -f \"%Y.%j.%H%M\""
#define		COMPRESSION	"STEIM2"
#define		BLOCK_SIZE	"4096"
#define		FORMAT		"SEED"
#define		NB_TOKENS	51
#define		HARD_PATH	"/data/09/stephane/DBMS_Population/Instrument_Responses/HT/NC/Hardware"
#define		DEBUG		0
#define		NETIGNORE	"net.ignore"
#define		STAIGNORE	"sta.ignore"

char *syntax[] = {
"%s version " VERSION " -- Build NCSN .tbl files from the .csv file.",
"%s  [-h] [-u file] <.csv file>",
"    where:",
"	-h		Help - prints syntax message.",
"	-u file		Specifies updated station file.",
NULL };

/************************************************************************/
/*  Data Structures.							*/
/************************************************************************/

/* Channel Comment Structure */
typedef struct {
		char	Network_Code[128];
		char    Site_Code[128];
		char    SEED_Channel_Code[128];
		char    Location_Code[128];
		char    Starting_Date[128];
		char    Ending_Date[128];
		char	Comments[128];
		} NC_Comment;

/* Channel Structure */
typedef struct {
		char	Network_Code[128];
		char    Site_Code[128];
                char    Location_Code[128];
                char    USGS_Channel_Code[128];
                char    Starting_Date[128];
                char    Ending_Date[128];
                char    Status[128];
		char	Status_Date[128];
                char    Operator[128];
                char    Latitude[128];
                char    Longitude[128];
                char    Elevation[128];
                char    Depth[128];
		char	Azimuth[128];
		char	Dip[128];
                char    Station_Name[128];
                char    Channel_Type[128];
                char    Sample_Rate[128];
                char    Sensor_Serial_Number[128];
                char    Pre_Amp_Serial_Number[128];
                char    Datalogger_Serial_Number[128];
		char	Component_Number[128];
		char	Pre_Amp_Channel_Number[128];
		char	Primary_Digit_Code[128];
		char	Primary_Pin_Number[128];
		char	Secondary_Digit_Code[128];
		char	Secondary_Pin_Number[128];
		char	FIR_Filter[128];
		char	Sensor_Type[128];
		char	Sensor_Units[128];
		char	Sensor_Sensitivity[128];
		char	Pre_Amp_Type[128];
		char	Pre_Amp_Gain[128];
		char	Primary_Datalogger[128];
		char	Secondary_Datalogger[128];
		char	Digitizer_Sensitivity[128];
		char	FM_Frequency_Channel[128];
		char	Tape_Track[128];
		char	Tape_Recorder[128];
		char	Field_Remarks[128];
		char	Sensor_Response[128];
		char	Pre_Amp_Response[128];
		char	Digitizer_1_Response[128];
		char	Digitizer_2_Response[128];
		char	Location_Reference[128];
		char	SEED_Channel_Code[128];
		char	Decimation_Filter[128];
		char	Decimation_Rate[128];
		char	USGS_Decimation_Channel[128];
		char	Decimation_Channel[128];
		char	Comments[128];
               } NC_Channel;

/* Station List Structure */
typedef struct	{
		 char	staname[16];	/* Network/Station name */
		} StaList;

/* Digitizer Module Structure */
typedef struct	{
		 int	dlt_chan;	/* Digitizer Module Number */
		 double	dlt_sens;	/* Digitizer Sensitivity */
		} Module;

/* Digitizer Board Structure */
typedef struct	{
		 char	dlt_btype[16];	/* Digitizer Board Type */
		 int	dlt_slot;	/* Digitizer Board Number */
		 char	dlt_dmsn[128];	/* Digitizer Serial Number */
		 int	dlt_nch;	/* Number Of Digitizer Modules */
		 Module	m[24];		/* Digitizer Modules */
		} Board;

/* Datalogger Structure */
typedef struct	{
		 char	dlt_type[32];	/* Datalogger Type */
		 char	dlt_sn[128];	/* Datalogger Serial Number */
		 char	dlt_firm[16];	/* Datalogger Firmware Number */
		 char	dlt_sdat[32];	/* Datalogger Start Date */
		 char	dlt_edat[32];	/* Datalogger End Date */
		 int	dlt_ndm;	/* Number Of Digitizer Boards */
		 Board	b[24];		/* Digitizer Boards */
		} Datalogger;

/* Station Structure */
typedef struct	{
		 char	DigiSN[128];	/* Digitizer Serial Number */
		 int	NbModule;	/* Number of Modules */
		 char	DIGICA[10000];	/* DIGICA lines */
		} digi;

typedef struct	{
		 char	DataSN[128];	/* Datalogger Serial Number */
		 int	NbChannel;	/* Number of Channels */
		 char	Data[5000];	/* DLOGCA & DLOGFC lines */
		 int	NbDigi;		/* Number of Digitizers */
		 digi	Digi[32];	/* Digitizers */
		} data;

typedef struct	{
		 char	SensorSN[128];	/* Sensor Serial Number */
		 char	Latitude[16];	/* Sensor Latitude */
		 char	Longitude[16];	/* Sensor Longitude */
		 char	Elevation[16];	/* Sensor Elevation */
		 char	Depth[16];	/* Sensor Depth */
		 int	NbComp;		/* Number of Components */
		 char	Senca[5000];	/* SENCA lines */
		} sensor;

typedef struct	{
		 char	PreAmpSN[128];	/* Pre-Amplifier Serial Number */
		 int	NbChan;		/* Number of Channels */
		 char	Pampca[5000];	/* PAMPCA lines */
		} preamp;

typedef struct	{
		 int	NbSensor;	/* Number of Sensors */
		 int	NbPreAmp;	/* Number of Pre-Amplifiers */
		 int	NbDigi;		/* Number of Digitizers */
		 int	NbData;		/* Number of Dataloggers */
		 int	NbChannel;	/* Number of Channels */
		 char	SDate[32];	/* Start Date */
		 char	EDate[32];	/* End Date */
		 sensor	Sensor[32];	/* Sensors */
		 preamp	PreAmp[32];	/* Pre-Amplifiers */
		 data	Data[32];	/* Dataloggers */
		} STINST;

/* Sensor Structure */
typedef struct	{
		 int	set_cmpnb;	/* Component Number */
		 char	set_chan;	/* Sensor Channel */
		 char	set_styp;	/* Sensor Type */
		 char	set_units[32];	/* Sensor Units */
		 double	set_sens;	/* Sensor Sensitivity */
		 double	set_freq;	/* Sensor Frequency */
		 int	set_nlin;	/* Number of lines in the response */
		 char	set_resp[1024];	/* Response */
		} SC;

typedef struct	{
		 char	set_sid[32];	/* Sensor Identification */
		 char	set_sn[128];	/* Sensor Serial Number */
		 char	set_sdat[32];	/* Sensor Starting Date */
		 char	set_edat[32];	/* Sensor Ending Date */
		 int	set_ncmp;	/* Number of components */
		 SC	set_comp[32];	/* Sensor's Components */
		} SensorHT;

/* Pre-Amplifier Structure */
typedef struct	{
		 int	pat_chan;		/* Channel Number */
		 int	pat_ngain;		/* Number of different gains */
		 char	pat_gsdat[32][32];	/* Gain Start Dates */
		 double	pat_gain[32];		/* Pre-Amplifier Sensitivities */
		 double	pat_freq;		/* Pre-Amplifier Frequency */
		 int	pat_nlin;		/* Number of lines in the response */
		 char	pat_resp[1024];		/* Response */
		} PA;

typedef struct	{
		 char	pat_paid[32];	/* Pre-Amplifier Identification */
		 char	pat_sn[128];	/* Pre-Amplifier Serial Number */
		 char	pat_sdat[32];	/* Pre-Amplifier Starting Date */
		 char	pat_edat[32];	/* Pre-Amplifier Ending Date */
		 int	pat_nchn;	/* Number of Channels */
		 PA	pat_chn[32];	/* Pre-Amplifier's Channels */
		} PreampHT;

/* Duplicate Channel Structure */
typedef struct {
		char    Site_Code[128];
                char    Location_Code[128];
                char    SEED_Channel_Code[128];
                char    Starting_Date[128];
                char    Ending_Date[128];
               } Dup;


/************************************************************************/
/*  External variables and symbols.					*/
/************************************************************************/
char *cmdname;			/* program name from command line.	*/

int	NbOperators	=	11;

char	*Operators[][2] = {
"CDM",	"California Division of Mines and Geology",
"CIT",	"Caltech",
"DWR",	"California Division of Water Resources",
"GSM",	"USGS Menlo Park",
"GSP",	"USGS Pasadena",
"GSPM",	"Formerly operated by Pasadena, now by Menlo Park",
"LLL",	"Lawrence Livermore Laboratories",
"NSMP",	"National Strong Motion Program",
"UCB",	"UC Berkeley",
"UNR",	"University of Nevada, Reno",
"USC",	"University of Southern California"};

int	NbTechRemarks	=	43;

char	*TechRemarks[][2] = {
"A",	"New installation",
"B",	"Replacement of unit",
"C",	"Re installation of original unit after repair",
"D",	"Change attenuation",
"E",	"Change Battery",
"F",	"Off frequency",
"G",	"VCO problems",
"H",	"Calibrator problems",
"I",	"Deviation change",
"J",	"Seismometer Problems JB Seismometer replacement",
"K",	"Telephone problems",
"L",	"Signal level problem",
"M",	"Radio Problems MB radio replacement",
"N",	"Antenna problems NB Antenna replacement",
"O",	"Landline Problems OB Landline replacement",
"P",	"Grounding problem",
"Q",	"Flooding problems",
"R",	"Routine maintainance,not covered by other codes",
"S",	"Seismometer leveling problems",
"T",	"Seismicly dead station",
"U",	"Amplifier problems",
"V",	"Vandalism",
"W",	"Summing amp problems WB Summing amp replacement",
"X",	"Noise Problems",
"Y",	"Stabilizer Problems",
"Z",	"Tape channel change",
"\\",	"Temporary removal from tape recorder",
"/",	"Permanent removal from tape recorder",
"1",	"Power Repair/Replacement",
"2",	"Data Logger Exchange",
"3",	"Firmware Changes",
"4",	"Data Recording and Transmission Parameter Change",
"5",	"GPS  Recording/Equipment Repair",
"6",	"Satellite Equipment Repair/Replacement",
"7",	"Earthworm Digitizing and Recording Repair/Replacement/Upgrade",
"8",	"Microwave Equipment Replacement/Repair",
"9",	"Telemetry Access Hardware Replacement",
"#",	"Site Electronic Noise/Interference",
"+",	"hardware change requires new channel/location",
"{",	"data recording begins",
"}",	"data recording ends",
"@",	"Site Move",
"!",	"Site Removal"};

/*  Global variables for the token list */
int     tokenc;
char    **tokenv;

/* Station List */
StaList		SL[2048];
int		NbSL = 0;	/* Number of Stations */

/* Channel list */
NC_Channel	CL[512];
int		NbCL = 0;	/* Number of Channels */
NC_Channel	CL2[512];
int		NbCL2 = 0;	/* Number of Unique Channels */

/* Station */
STINST		stinst;

/* Channel Comment */
NC_Comment	ComOperator;
NC_Comment	ComFM;
NC_Comment	ComTrack;
NC_Comment	ComRecorder;

/* Duplicate Channels List */
Dup	DupCha[512];
int	NbDupCha = 0;		/* Number of Duplicate Channels */
char	DupCha2[50][50];
int	NbDupCha2 = 0;		/* Number of Duplicate Channels in .tbl File */

/* Ignore lists */
char	IgnoreNet[255][255];
char	IgnoreSta[255][255];
char	UpdatedSta[2048][255];
int	NbIgnoreNet = 0;
int	NbIgnoreSta = 0;
int	NbUpdatedSta = 0;

/************************************************************************/
/*  Functions Declarations.						*/
/************************************************************************/
int     split_inline (char *line, char delim);
int     split_copy  (char *line, char delim);
int     reset_token (int *pargc, char ***pargv, int dup);
int     push_token (int *pargc, char ***pargv, char *p, int dup);
char    *strndup (char *p, int n);

/************************************************************************/
/*  print_syntax:							*/
/*	Print the syntax description of program.			*/
/************************************************************************/
int print_syntax2
   (char	*cmd,		/* program name.			*/
    char	*syntax[],	/* syntax array.			*/
    FILE	*fp)		/* FILE ptr for output.			*/
{
    int i;
    for (i=0; syntax[i] != NULL; i++) {
	fprintf (fp, syntax[i], cmd);
	fprintf (fp, "\n");
    }
    return (0);
}

/************************************************************************/
/*  stringcompare:							*/
/*	Function to compare strings.					*/
/************************************************************************/
static  int stringcompare (const void *i, const void *j)
{
 return (strcmp (i, j));
}

/************************************************************************/
/*  CompareInterval:							*/
/*	Function to compare two time intervals.				*/
/*	Return 1 if intervals overlap, 0 otherwise.			*/
/************************************************************************/
int	CompareInterval (char s1[128], char e1[128], char s2[128], char e2[128])
{
 char	ns1[13], ne1[13], ns2[13], ne2[13];
 
 /* Converting Times */
 sprintf (ns1, "%.4s%.2s%.2s%.2s%.2s", s1+6, s1, s1+3, s1+11, s1+14);
 sprintf (ns2, "%.4s%.2s%.2s%.2s%.2s", s2+6, s2, s2+3, s2+11, s2+14);
 sprintf (ne1, "%.4s%.2s%.2s%.2s%.2s", e1+6, e1, e1+3, e1+11, e1+14);
 sprintf (ne2, "%.4s%.2s%.2s%.2s%.2s", e2+6, e2, e2+3, e2+11, e2+14);
 
 /* Compare Intervals */
 if ((strcmp (ns1, ne2) < 0) && (strcmp (ne1, ns2) > 0))
	{
	 return (1);
	}
 else
 	{
 	 return (0);
 	}
}


/************************************************************************/
/*  Remove_Blank:							*/
/*	Function to remove trailing blanks in a string.			*/
/************************************************************************/
char*   Remove_Blank (s)
char    *s;
{
 char   *p = s + strlen (s);
 while (--p >= s)
        if (*p == ' ')
                *p = '\0';
        else    break;
 return (s);
}

/************************************************************************/
/*  Replace_Blank:                                                       */
/*      Function to replace blanks in a string.                         */
/************************************************************************/
char*   Replace_Blank (s)
char    *s;
{
 char   *p = s + strlen (s);
 while (--p >= s)
        if (*p == ' ')
                *p = '-';
 return (s);
}

/************************************************************************/
/*  GetOperator:							*/
/*	Function to retrieve an operator.				*/
/************************************************************************/
char	*GetOperator (char	o[128])
{
 int	i;

 for (i=0;i < NbOperators; i++)
	if (!strcmp (o, Operators[i][0]))
		return (Operators[i][1]);

 return (o);
}

/************************************************************************/
/*  GetTechRemark:							*/
/*	Function to retrieve a field remark.				*/
/************************************************************************/
char	*GetTechRemark (char	r)
{
 int	i;
 static char	rr[3];


 rr[0] = r;
 rr[1] = '\0';

 for (i=0;i < NbTechRemarks; i++)
	if (!strcmp (rr, TechRemarks[i][0]))
		return (TechRemarks[i][1]);

 return (rr);
}

/************************************************************************/
/*  ConvertDate:							*/
/*	Function to convert a date.					*/
/************************************************************************/
char	*ConvertDate (char	d[255])
{
 FILE  		*cmd_ptr;
 char 	 	cmd_line[256];
 static char	cd[255];

 if (!strcmp (d, "Present"))
	strcpy (cd, "2599.366.2359");
 else
	{
	 sprintf (cmd_line , "%s %.4s/%.2s/%.2s,%.2s:%.2s", CALDATE, d+6, d, d+3, d+11, d+14);

 	 if ((cmd_ptr = popen (cmd_line,"r")) == NULL)
		exit (-1);

	 if ((fgets (cd, 254, cmd_ptr)) == NULL)
		exit (-2);

	 pclose (cmd_ptr);
	}

 if (cd[strlen (cd) - 1] == '\n')
	cd[strlen (cd) - 1] = '\0';

 return (cd);
}

/************************************************************************/
/*  ReadIgnore:								*/
/*	Function to read the ignore lists.				*/
/************************************************************************/
void	ReadIgnore ()
{
 FILE	*fnet, *fsta;
 char	snet[255], ssta[255];
 char	line[255];
 char	tmp[255];


 strcpy (snet, NETIGNORE);
 strcpy (ssta, STAIGNORE);

 if ((fnet = fopen (snet, "rt")) == NULL)
	{
	 printf ("Error: Couldn't open file %s\n", snet);
	 exit (-1);
	}

 if ((fsta = fopen (ssta, "rt")) == NULL)
	{
	 printf ("Error: Couldn't open file %s\n", ssta);
	 exit (-1);
	}

 fgets (line, 254, fnet);
 while (!feof (fnet))
	{
	 sscanf (line, "%s", tmp);
	 strcpy (IgnoreNet[NbIgnoreNet++], tmp);

	 fgets (line, 254, fnet);
	}

 fgets (line, 254, fsta);
 while (!feof (fsta))
	{
	 sscanf (line, "%s", tmp);
	 strcpy (IgnoreSta[NbIgnoreSta++], tmp);

	 fgets (line, 254, fsta);
	}

 fclose (fnet);
 fclose (fsta);
}

/************************************************************************/
/*  NetIsIgnored:							*/
/*	Function to test if a network is ignored.			*/
/************************************************************************/
int	NetIsIgnored (char	net[128])
{
 int	i;


 for (i=0;i < NbIgnoreNet; i++)
	{
	 if (!strcmp (net, IgnoreNet[i]))
		return (1);
	}

 return (0);
}

/************************************************************************/
/*  StaIsIgnored:							*/
/*	Function to test if a station is ignored.			*/
/************************************************************************/
int	StaIsIgnored (char	net[128], char	sta[128])
{
 int	i;
 char	netsta[256];


 sprintf (netsta, "%s.%s", net, sta);

 for (i=0;i < NbIgnoreSta; i++)
	{
	 if (!strcmp (netsta, IgnoreSta[i]))
		return (1);
	}

 return (0);
}

/************************************************************************/
/*  ReadUpdated:							*/
/*	Function to read the update list.				*/
/************************************************************************/
void	ReadUpdated (char	*ssta)
{
 FILE	*fsta;
 char	line[255];
 char	tmp[255];
 char	tmp2[255];


 if ((fsta = fopen (ssta, "rt")) == NULL)
	{
	 printf ("Error: Couldn't open file %s\n", ssta);
	 exit (-1);
	}

 fgets (line, 254, fsta);
 while (!feof (fsta))
	{
	 sscanf (line, "%s %s", tmp, tmp2);
	 sprintf (UpdatedSta[NbUpdatedSta++], "%s.%s", tmp, tmp2);

	 fgets (line, 254, fsta);
	}

 fclose (fsta);
}



/************************************************************************/
/*  main:   main program.						*/
/************************************************************************/
main (int	argc, char	*argv[])
{
 FILE		*fcsv;		/* .csv File Descriptor */
 FILE		*fver;		/* Version File Descriptor */
 FILE		*fverold;	/* Old Version File Descriptor */
 FILE		*ftmp;		/* Temporary File Descriptor */
 char		ncsv[255];	/* .csv File Name */
 char		nver[255];	/* Version File Name */
 char		nverold[255];	/* Old Version File Name */
 char		ntmp[255];	/* Temporary File Name */
 char		nresp[255];	/* Response File Name */
 char		line[1024];	/* Current line */
 char		cline[1024];	/* Copy of current line */
 char		nline[1024];	/* New line */
 int		n;
 int		i,j,k,l,m,x,y,z;/* Loops */
 int		xx, zz;		/* More Loops */
 char		tmploc[5];	/* Temporary buffer */
 char		tmpfir[128];	/* Temporary buffer */

 char		old_sta[12];	/* Old Site Code */
 char		old_cha[12];	/* Old SEED Channel Name */
 char		old_loc[12];	/* Old SEED Location Code */
 char		old_stime[12];	/* Old Start Time */

 char		Version[255];	/* Current Version */
 char		NVersion[255];	/* New Version */
 char		CompDate[255];	/* Complete date */
 int		FlagVersion;	/* Version Flag */
 int		StaKey;		/* Station Key */

 int		Flag_Channel;	/* Channel Flag */
 FILE		*ferrortbl;	/* Error .tbl Log File Descriptor */
 FILE		*fsensor;	/* Sensor.tbl File Descriptor */
 FILE		*fpreamp;	/* Preamp.tbl File Descriptor */
 FILE		*fdatlog;	/* Datlog.tbl File Descriptor */
 FILE		*fsta;		/* Sta.tbl File Descriptor */
 char		nsta[255];	/* .tbl Station File Name */

 char		set_sid[64];	/* Sensor Identification */
 char		set_sdat[64];	/* Sensor Start Date */
 char		set_edat[64];	/* Sensor End Date */
 char		set_styp;	/* Sensor Type */
 char		set_units[64];	/* Sensor Units */
 char		set_sens[64];	/* Sensor Sensitivity */

 char		pat_sdat[64];	/* Pre-amplifier Start Date */
 char		pat_edat[64];	/* Pre-amplifier End Date */

 char		*timespantmp;	/* Temporary Time Span Array */
 char		*timespan;	/* Time Span Array */
 int		NbTS;		/* Number of Time Spans */
 char		ts[14];		/* Time Span */

 int		Flag_Station;	/* Station Flag */
 int		Flag_Datlog;	/* Datalogger Flag */
 int		Flag_Snsr;	/* Sensor Flag */
 int		Flag_Prmp;	/* Pre-Amplifier Flag */
 Datalogger	dtlgr[32];	/* Dataloggers */
 SensorHT	snsrHT[32];	/* Sensors */
 PreampHT	prmpHT[32];	/* Pre-Amplifiers */
 int		NbData;		/* Number of dataloggers */
 int		NbSnsr;		/* Number of sensors */
 int		NbPrmp;		/* Number of Pre-Amplifiers */
 int		BoardNb;	/* Board Number */
 int		ModuleNb;	/* Module Number */
 int		CmpnntNb;	/* Component Number */
 int		ChnnlNb;	/* Channel Number */
 char		tmpline[255];	/* Temporary Line */
 char		tmpline2[255];	/* Temporary Line */
 int		pin_tmp;	/* Temporary Pin Number */

 struct stat	buf;		/* File Attributes */

 FILE		*fresp;		/* Response File Descriptor */
 FILE		*frem;		/* .rem File Descriptor */
 char		nrem[255];	/* .rem File Name */

 int		FlagOperator;
 int		FlagFM;
 int		FlagTrack;
 int		FlagRecorder;

 int		nblines;	/* Number of logical channel lines */

 int		err;
 int		FlagU = 0;	/* Flag Updated */
 char		tmp[255];

 /* Variables needed for getopt. */
 extern char	*optarg;
 extern int	optind, opterr;
 int		c;
 char	*p;

 cmdname = ((p = strrchr(*argv,'/')) != NULL) ? ++p : *argv;
 /*	Parse command line options.					*/
 while ( (c = getopt(argc,argv,"hu:")) != -1) switch (c) {
	case '?':
	case 'h':	print_syntax(cmdname,syntax,info); exit(0); break;
	case 'u':	FlagU = 1; ReadUpdated (optarg); break;
	default:	fprintf (info, "Unknown option: -%c\n", c); exit(0);
 }
 /*	Skip over all options and their arguments.			*/
 argv = &(argv[optind]);
 argc -= optind;

 /* Testing parameters */
 if (argc != 1)
	{
	 print_syntax(cmdname,syntax,info);
	 exit (0);
	}

 /* Read ignore lists */
 ReadIgnore ();

 /* Opening .csv file */
 sprintf (ncsv, "%s", argv[0]);
 if ((fcsv = fopen (ncsv, "rt")) == NULL)
	{
	 printf ("\nError: Couldn't open file %s.\n", ncsv);
	 exit (-4);
	}

 /* Updated List? */
 if (FlagU == 1)
	{
	 for (i=0; i < NbUpdatedSta; i++)
		 strcpy (SL[NbSL++].staname, UpdatedSta[i]);
	}
else	{

 /************************************************************************/
 /*                                                                      */
 /* Phase I:                                                             */
 /*          - Read current version in the version file.                 */
 /*          - Browse the .csv file and create a list of stations        */
 /*            that have a status date later that the current version    */
 /*            date. Also, find the new version date which is the latest */
 /*            date in the status field.                                 */
 /*          - If there is a new version, write it and backup the old    */
 /*            version date.                                             */
 /*                                                                      */
 /************************************************************************/

 printf ("\n\n Phase I ...");

 /* Opening Version File */
 sprintf (nver, "%s", VERSION_FILE);
 if ((fver = fopen (nver, "r+t")) == NULL)
	{
	 printf ("\nError: Couldn't open file %s.\n", nver);
	 exit (-3);
	}

 /* Reading Version */
 fscanf (fver, "%s", Version);
 printf ("\n\t Current Version: \t%s\n", Version);
 strcpy (NVersion, Version);
 FlagVersion = 0;

 /* Skip first line of .csv file */
 fgets (line, 1023, fcsv);

 /* Reading second line */
 fgets (line, 1023, fcsv);

 while (!feof (fcsv))
	{
	 line[strlen (line) - 1] = '\0';
	 n = 0;

	 n = split_inline (line, ',');

	 if (n != NB_TOKENS)
		{
		 printf ("\n Number of tokens: %d (%s.%s)", n, tokenv[1], tokenv[3]);

		 exit (-5);
		}

	 for (i=0;i<NB_TOKENS;i++)
		Remove_Blank (tokenv[i]);

	 /* Convert Complete Date */
	 sprintf (CompDate, "%.4s/%.2s/%.2s", tokenv[7] + 6, tokenv[7], tokenv[7] + 3);

	 /* New/Updated Station? */
	 if ((strcmp (Version, CompDate) < 0) && (!strcmp (tokenv[6], "C")))
		{
		 FlagVersion = 1;

		 /* Is station already in station list? */
		 StaKey = -1;
		 sprintf (tmp, "%s.%s", tokenv[0], tokenv[1]);
		 for (i=0;i<NbSL;i++)
			if (!strcmp (SL[i].staname, tmp))
				StaKey = i;

		 if ((StaKey == (-1)) && !(NetIsIgnored(tokenv[0])) && !(StaIsIgnored(tokenv[0], tokenv[1])))
			{
			 printf ("\n\t Station [%s.%s] is new or updated.", tokenv[0], tokenv[1]);

			 sprintf (SL[NbSL++].staname, "%s.%s", tokenv[0], tokenv[1]);

			 sprintf (ntmp, "%s.%s.tmp", tokenv[0], tokenv[1]);
			 if ((ftmp = fopen (ntmp, "wt")) == NULL)
				{
				 printf ("\nError: Couldn't open file %s.\n", ntmp);
				 exit (-6);
				}

			 /* Closing station file */
			 fclose (ftmp);
			}

		 if (strcmp (NVersion, CompDate) < 0)
			strcpy (NVersion, CompDate);
		}

	 reset_token (&tokenc, &tokenv, 0);

	 /* Reading next line */
	 fgets (line, 1023, fcsv);
	}

 /* New Version? */
 if (FlagVersion)
	{
	 /* Opening Old Version File */
	 sprintf (nverold, "%s.old", VERSION_FILE);
	 if ((fverold = fopen (nverold, "wt")) == NULL)
		{
		 printf ("\nError: Couldn't open file %s.\n", nverold);
		 exit (-7);
		}

	 /* Writing Old Version */
	 fprintf (fverold, "%s\n", Version);

	 /* Closing Old Version File */
	 fclose (fverold);

	 /* Writing New Version */
	 rewind (fver);
	 fprintf (fver, "%s\n", NVersion);

	 printf ("\n\n\t New Version: \t\t%s\n", NVersion);
	}

 /* Closing New Version File */
 fclose (fver);

 /* No new/updated stations? */
 if (NbSL == 0)
	{
	 printf ("\n\t No new or updated stations.\n\n");

	 exit (0);
	}
	}

 /************************************************************************/
 /*                                                                      */
 /* Phase II:                                                            */
 /*           - Rewind the .csv file.                                    */
 /*           - Browse the .csv file and for each station in the station */
 /*             list, copy its entries from the .csv file to the file    */
 /*             "sta.tmp".                                               */
 /*                                                                      */
 /************************************************************************/

 printf ("\n\n Phase II ...");

 /* Rewinding .csv file */
 rewind (fcsv);

 /* Skip first line */
 fgets (line, 1023, fcsv);

 /* Reading second line */
 fgets (line, 1023, fcsv);

 while (!feof (fcsv))
	{
	 strcpy (cline, line);
	 line[strlen (line) - 1] = '\0';
	 n = 0;

	 n = split_inline (line, ',');

	 if (n != NB_TOKENS)
		{
		 printf ("\n Number of tokens: %d (%s.%s)", n, tokenv[1], tokenv[3]);

		 exit (-8);
		}

	 for (i=0;i<NB_TOKENS;i++)
		Remove_Blank (tokenv[i]);

	 /* Is station in station list? */
	 StaKey = -1;
	 sprintf (tmp, "%s.%s", tokenv[0], tokenv[1]);
	 for (i=0;i<NbSL;i++)
		if (!strcmp (SL[i].staname, tmp))
			StaKey = i;

	 if ((StaKey != (-1)) && (!strcmp (tokenv[6], "C")))
		{
		 sprintf (ntmp, "%s.%s.tmp", tokenv[0], tokenv[1]);
		 if ((ftmp = fopen (ntmp, "at")) == NULL)
			{
			 printf ("\nError: Couldn't open file %s.\n", ntmp);
			 exit (-9);
			}

		 printf ("\n\t Writing entry for station [%s.%s].", tokenv[0], tokenv[1]);
		 /*fprintf (ftmp, "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n", 
				tokenv[0], tokenv[1], tokenv[2], tokenv[4], tokenv[5], tokenv[9], tokenv[10],
				tokenv[11], tokenv[12], tokenv[13], tokenv[14], tokenv[15], tokenv[17], tokenv[18],
				tokenv[19], tokenv[20], tokenv[21], tokenv[22], tokenv[24], tokenv[27], tokenv[45]);*/

		 fprintf (ftmp, "%s", cline); 

		 fclose (ftmp);
		}

	 reset_token (&tokenc, &tokenv, 0);

	 /* Reading next line */
	 fgets (line, 1023, fcsv);
	}

 /* Closing .csv file */
 fclose (fcsv);

 /************************************************************************/
 /*                                                                      */
 /* Phase III:                                                           */
 /*            - For each station in the station list, create from the   */
 /*              "sta.tmp" file the files "sta.rem", "sta.tbl",          */
 /*              "sensor.tbl", "datlog.tbl" and eventually "preamp.tbl". */
 /*                                                                      */
 /************************************************************************/

 printf ("\n\n Phase III ...");

 /* Opening error.NC.log file */
 if ((ferrortbl = fopen ("error.NC.log", "wt")) == NULL)
	{
	 printf ("\nError: Couldn't open file error.NC.log.\n");
	 exit (-10);
	}

 /* Opening sensor.NC.tbl file */
 if ((fsensor = fopen ("sensor.NC.tbl", "wt")) == NULL)
	{
	 printf ("\nError: Couldn't open file sensor.NC.tbl.\n");
	 exit (-11);
	}

 /* Opening preamp.NC.tbl file */
 if ((fpreamp = fopen ("preamp.NC.tbl", "wt")) == NULL)
	{
	 printf ("\nError: Couldn't open file preamp.NC.tbl.\n");
	 exit (-12);
	}

 /* Opening datlog.NC.tbl file */
 if ((fdatlog = fopen ("datlog.NC.tbl", "wt")) == NULL)
	{
	 printf ("\nError: Couldn't open file datlog.NC.tbl.\n");
	 exit (-13);
	}

 /* Browsing stations */
 for (i=0;i<NbSL;i++)
	{
	 Flag_Station = 0;
	 FlagOperator = 0;
	 FlagFM = 0;
	 FlagTrack = 0;
	 FlagRecorder = 0;

	 strcpy (ComRecorder.Network_Code, "");
	 strcpy (ComRecorder.Site_Code, "");
	 strcpy (ComRecorder.SEED_Channel_Code, "");
	 strcpy (ComRecorder.Location_Code, "");

	 printf ("\n\t Creating .tbl & .rem files for station [%s].", SL[i].staname);

	 NbCL = 0;
	 NbDupCha = 0;

	 /* Opening tmp station file */
	 sprintf (ntmp, "%s.tmp", SL[i].staname);
	 if ((ftmp = fopen (ntmp, "rt")) == NULL)
		{
		 printf ("\nError: Couldn't open file %s.\n", ntmp);
		 exit (-14);
		}

	 /* Opening tbl station file */
	 sprintf (nsta, "%s.tbl", SL[i].staname);
	 if ((fsta = fopen (nsta, "wt")) == NULL)
		{
		 printf ("\nError: Couldn't open file %s.\n", nsta);
		 exit (-15);
		}

	 /* Opening rem station file */
	 sprintf (nrem, "%s.rem", SL[i].staname);
	 if ((frem = fopen (nrem, "wt")) == NULL)
		{
		 printf ("\nError: Couldn't open file %s.\n", nsta);
		 exit (-16);
		}


	 fgets (line, 1023, ftmp);

	 while (!feof (ftmp))
		{
		 strcpy (cline, line);
		 line[strlen (line) - 1] = '\0';
		 n = 0;

		 n = split_inline (line, ',');

		 if (n != NB_TOKENS)
			{
			 printf ("\n Number of tokens: %d (%s.%s)", n, tokenv[1], tokenv[3]);

			 exit (-17);
			}

		 for (j=0;j<NB_TOKENS;j++)
			{
			 Remove_Blank (tokenv[j]);
			}

		   /************************/
		  /* Testing Channel Data */
		 /************************/
		 Flag_Channel = 0;

		 if ((!strcmp (tokenv[0], "")) || (!strcmp (tokenv[0], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[0], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Network Code missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 else if (NetIsIgnored(tokenv[0]))
			{
			 Flag_Channel = 1;
			}
		 if ((!strcmp (tokenv[1], "")) || (!strcmp (tokenv[1], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[1], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Site Code missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 else if (StaIsIgnored(tokenv[0], tokenv[1]))
			{
			 Flag_Channel = 1;
			}
		 /*if (!strcmp (tokenv[2], ""))
			{
			 Flag_Channel = 1;
			 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Location Code missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}*/
		 if ((!strcmp (tokenv[4], "")) || (!strcmp (tokenv[4], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[4], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Starting Date missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[5], "")) || (!strcmp (tokenv[5], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[5], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Ending Date missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[9], "")) || (!strcmp (tokenv[9], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[9], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Latitude missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[10], "")) || (!strcmp (tokenv[10], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[10], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Longitude missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[11], "")) || (!strcmp (tokenv[11], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[11], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Elevation missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[12], "")) || (!strcmp (tokenv[12], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[12], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Depth missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[13], "")) || (!strcmp (tokenv[13], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[13], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Azimuth missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[14], "")) || (!strcmp (tokenv[14], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[14], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Dip missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[15], "")) || (!strcmp (tokenv[15], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[15], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Station Name missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[16], "")) || (!strcmp (tokenv[16], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[16], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Channel Type missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[17], "")) || (!strcmp (tokenv[17], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[17], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sample Rate missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 else if (atof (tokenv[17]) == 0.)
			{
			 Flag_Channel = 1;
			}
		 if ((!strcmp (tokenv[18], "")) || (!strcmp (tokenv[18], "-")) || (!strcmp (tokenv[18], "unknown")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[18], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Serial Number missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[20], "")) || (!strcmp (tokenv[20], "-")) || (!strcmp (tokenv[20], "unknown")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[20], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Datalogger Serial Number missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[21], "")) || (!strcmp (tokenv[21], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[21], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Component Number missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 else if (atoi (tokenv[21]) < 0)
			{
			 Flag_Channel = 1;
			 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Invalid Sensor Component (%s).", tokenv[0], tokenv[1], tokenv[45], tokenv[4], tokenv[21]);
			}
		 if ((!strcmp (tokenv[24], "")) || (!strcmp (tokenv[24], "-")))
			{
			 /* Copying Secondary Pin number */
			 strcpy (tokenv[24], tokenv[26]);

			 if ((!strcmp (tokenv[24], "")) || (!strcmp (tokenv[24], "-")))
				{
			 	 Flag_Channel = 1;

			 	 if (!strcmp (tokenv[24], ""))
			 		fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Primary & Secondary Pin Numbers missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
				}
			}
		 else if ((atoi (tokenv[24]) < 0) || (atoi (tokenv[24]) > 640))
			{
			 Flag_Channel = 1;
			 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Invalid Pin Number (%s).", tokenv[0], tokenv[1], tokenv[45], tokenv[4], tokenv[24]);
			}
		 if ((!strcmp (tokenv[27], "")) && (!strcmp (tokenv[27], "-")) && (!strcmp (tokenv[27], "~")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[27], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: FIR Filter Name missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 /*if ((!strcmp (tokenv[28], "")) || (!strcmp (tokenv[28], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[28], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Type missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[29], "")) || (!strcmp (tokenv[29], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[29], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Units missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 if ((!strcmp (tokenv[30], "")) || (!strcmp (tokenv[30], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[30], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Sensitivity missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}*/
		 if ((!strcmp (tokenv[33], "")) || (!strcmp (tokenv[33], "-")))
			{
			 /* Copying Secondary Datalogger */
			 strcpy (tokenv[33], tokenv[34]);

			 if ((!strcmp (tokenv[33], "")) || (!strcmp (tokenv[33], "-")))
				{
			 	 Flag_Channel = 1;

			 	 if (!strcmp (tokenv[33], ""))
			 		fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Primary Datalogger missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
				}
			}
		 if ((!strcmp (tokenv[35], "")) || (!strcmp (tokenv[35], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[35], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Digitizer Sensitivity missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
			
		 if ((!strcmp (tokenv[40], "")) || (!strcmp (tokenv[40], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[40], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Response missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
		 else
		 	{
		 	 FILE	*tmpf;
		 	 char	tmps[256];
		 	 
		 	 sprintf (tmps, "%s/%s", HARD_PATH, tokenv[40]);
		 	 
		 	 if ((tmpf = fopen (tmps, "rt")) == NULL)
		 	 	{
		 	 	 Flag_Channel = 1;
		 	 	 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Sensor Response Class (%s) missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4], tokenv[40]);
		 	 	}
		 	 else	fclose (tmpf);
		 	}
		 	
		 if ((strcmp (tokenv[41], "")) && (strcmp (tokenv[41], "-")))
		 	{
		 	 FILE	*tmpf;
		 	 char	tmps[256];
		 	 
		 	 sprintf (tmps, "%s/%s", HARD_PATH, tokenv[41]);
		 	 
		 	 if ((tmpf = fopen (tmps, "rt")) == NULL)
		 	 	{
		 	 	 Flag_Channel = 1;
		 	 	 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Preamplifier Response Class (%s) missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4], tokenv[41]);
		 	 	}
		 	 else	fclose (tmpf);
		 	}
			
		 if ((!strcmp (tokenv[45], "")) || (!strcmp (tokenv[45], "-")))
			{
			 Flag_Channel = 1;

			 if (!strcmp (tokenv[45], ""))
			 	fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: SEED Channel Code missing.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
			}
			
		 if (Flag_Channel == 0)
		 	{
		 	 for (z=0;z<NbDupCha;z++)
		 	 	{
		 	 	 if (	(!strcmp (DupCha[z].Site_Code, tokenv[1])) &&
		 	 	 	(!strcmp (DupCha[z].SEED_Channel_Code, tokenv[45])) &&
		 	 	 	(!strcmp (DupCha[z].Location_Code, tokenv[2]))
		 	 	    )
		 	 	    	{
		 	 	    	 if (CompareInterval (	DupCha[z].Starting_Date,
		 	 	    	 			DupCha[z].Ending_Date,
		 	 	    	 			tokenv[4],
		 	 	    	 			tokenv[5]) == 1)
		 	 	    	 	{
		 	 	    	 	 Flag_Channel = 1;
		 	 	    	 	 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Duplicate Channel Codes.", tokenv[0], tokenv[1], tokenv[45], tokenv[4]);
		 	 	    	 	}
		 	 	    	}

				 /* Testing Decimated Channel */
				 if (   (!strcmp (DupCha[z].Site_Code, tokenv[1])) &&
                                        (!strcmp (DupCha[z].SEED_Channel_Code, tokenv[48])) &&
                                        (!strcmp (DupCha[z].Location_Code, tokenv[2]))
                                    )
                                        {
                                         if (CompareInterval (  DupCha[z].Starting_Date,
                                                                DupCha[z].Ending_Date,
                                                                tokenv[4],
                                                                tokenv[5]) == 1)
                                                {
                                                 Flag_Channel = 1;
                                                 fprintf (ferrortbl, "\nError [%s.%s.%s.%s]: Duplicate Channel Codes.", tokenv[0], tokenv[1], tokenv[48], tokenv[4]);
                                                }
                                        }
		 	 	}
		 	}

		 if (Flag_Channel == 0)
			{
			 strcpy (DupCha[NbDupCha].Site_Code, tokenv[1]);
			 strcpy (DupCha[NbDupCha].SEED_Channel_Code, tokenv[45]);
			 strcpy (DupCha[NbDupCha].Location_Code, tokenv[2]);
			 strcpy (DupCha[NbDupCha].Starting_Date, tokenv[4]);
			 strcpy (DupCha[NbDupCha++].Ending_Date, tokenv[5]);

			 if (strcmp (tokenv[48], "") && strcmp (tokenv[48], "-"))
				{
				 strcpy (DupCha[NbDupCha].Site_Code, tokenv[1]);
                         	 strcpy (DupCha[NbDupCha].SEED_Channel_Code, tokenv[48]);
                         	 strcpy (DupCha[NbDupCha].Location_Code, tokenv[2]);
                         	 strcpy (DupCha[NbDupCha].Starting_Date, tokenv[4]);
                         	 strcpy (DupCha[NbDupCha++].Ending_Date, tokenv[5]);
				}
			 
			
			   /******************************/
			  /* Building Channel Structure */
			 /******************************/
			 strcpy (CL[NbCL].Network_Code, 		tokenv[0]);
			 strcpy (CL[NbCL].Site_Code, 			tokenv[1]);
			 if (strlen (tokenv[2]) != 0)
			 	strcpy (CL[NbCL].Location_Code, 	tokenv[2]);
			 else	strcpy (CL[NbCL].Location_Code,		"  ");
			 strcpy (CL[NbCL].USGS_Channel_Code, 		tokenv[3]);
			 strcpy (CL[NbCL].Starting_Date, 		ConvertDate (tokenv[4]));
			 strcpy (CL[NbCL].Ending_Date, 			ConvertDate (tokenv[5]));
			 strcpy (CL[NbCL].Status, 			tokenv[6]);
			 strcpy (CL[NbCL].Status_Date, 			tokenv[7]);
			 strcpy (CL[NbCL].Operator, 			tokenv[8]);
			 strcpy (CL[NbCL].Latitude, 			tokenv[9]);
			 strcpy (CL[NbCL].Longitude, 			tokenv[10]);
			 strcpy (CL[NbCL].Elevation, 			tokenv[11]);
			 strcpy (CL[NbCL].Depth, 			tokenv[12]);
			 strcpy (CL[NbCL].Azimuth, 			tokenv[13]);
			 strcpy (CL[NbCL].Dip, 				tokenv[14]);
			 strcpy (CL[NbCL].Station_Name, 		tokenv[15]);
			 strcpy (CL[NbCL].Channel_Type, 		tokenv[16]);
			 strcpy (CL[NbCL].Sample_Rate, 			tokenv[17]);
			 if (strlen (tokenv[18]) != 0)
				sprintf (CL[NbCL].Sensor_Serial_Number, "NC.%s.%s", Replace_Blank(tokenv[28]), tokenv[18]);
			 else	strcpy (CL[NbCL].Sensor_Serial_Number, "");
			 if (!strcmp (tokenv[18], "-"))
				strcpy (CL[NbCL].Sensor_Serial_Number, "");
			 if (strlen (tokenv[19]) != 0)
				sprintf (CL[NbCL].Pre_Amp_Serial_Number, "NC.%s.%s", Replace_Blank(tokenv[31]), tokenv[19]);
			 else	strcpy (CL[NbCL].Pre_Amp_Serial_Number, "");
			 if (!strcmp (tokenv[19], "-")) 
                                strcpy (CL[NbCL].Pre_Amp_Serial_Number, "");
			 if (strlen (tokenv[20]) != 0)
				sprintf (CL[NbCL].Datalogger_Serial_Number, "NC.%s.%s", Replace_Blank(tokenv[33]), tokenv[20]);
			 else	strcpy (CL[NbCL].Datalogger_Serial_Number, "");
			 /*if (!strcmp (tokenv[20], "-")) 
                                strcpy (CL[NbCL].Datalogger_Serial_Number, "");*/
			 strcpy (CL[NbCL].Component_Number, 		tokenv[21]);
			 strcpy (CL[NbCL].Pre_Amp_Channel_Number, 	tokenv[22]);
			 strcpy (CL[NbCL].Primary_Digit_Code, 		tokenv[23]);
			 strcpy (CL[NbCL].Primary_Pin_Number, 		tokenv[24]);
			 strcpy (CL[NbCL].Secondary_Digit_Code, 	tokenv[25]);
			 strcpy (CL[NbCL].Secondary_Pin_Number, 	tokenv[26]);
			 strcpy (CL[NbCL].FIR_Filter, 			tokenv[27]);
			 strcpy (CL[NbCL].Sensor_Type, 			tokenv[28]);
			 strcpy (CL[NbCL].Sensor_Units, 		tokenv[29]);
			 strcpy (CL[NbCL].Sensor_Sensitivity, 		tokenv[30]);
			 strcpy (CL[NbCL].Pre_Amp_Type, 		tokenv[31]);
			 strcpy (CL[NbCL].Pre_Amp_Gain, 		tokenv[32]);
			 strcpy (CL[NbCL].Primary_Datalogger, 		tokenv[33]);
			 strcpy (CL[NbCL].Secondary_Datalogger, 	tokenv[34]);
			 strcpy (CL[NbCL].Digitizer_Sensitivity, 	tokenv[35]);
			 strcpy (CL[NbCL].FM_Frequency_Channel, 	tokenv[36]);
			 strcpy (CL[NbCL].Tape_Track, 			tokenv[37]);
			 strcpy (CL[NbCL].Tape_Recorder, 		tokenv[38]);
			 strcpy (CL[NbCL].Field_Remarks, 		tokenv[39]);
			 strcpy (CL[NbCL].Sensor_Response, 		tokenv[40]);
			 strcpy (CL[NbCL].Pre_Amp_Response, 		tokenv[41]);
			 strcpy (CL[NbCL].Digitizer_1_Response, 	tokenv[42]);
			 strcpy (CL[NbCL].Digitizer_2_Response, 	tokenv[43]);
			 strcpy (CL[NbCL].Location_Reference, 		tokenv[44]);
			 strcpy (CL[NbCL].SEED_Channel_Code, 		tokenv[45]);
			 strcpy (CL[NbCL].Decimation_Filter, 		tokenv[46]);
			 strcpy (CL[NbCL].Decimation_Rate, 		tokenv[47]);
			 strcpy (CL[NbCL].USGS_Decimation_Channel, 	tokenv[48]);
			 strcpy (CL[NbCL].Decimation_Channel,		tokenv[49]);
			 strcpy (CL[NbCL].Comments, 			tokenv[50]);
			 NbCL++;


			   /*************************/
			  /* Building sta.rem file */
			 /*************************/
if (DEBUG) printf ("\n%s %s %s %s %s %s:", CL[NbCL-1].Network_Code, CL[NbCL-1].Site_Code, CL[NbCL-1].SEED_Channel_Code, CL[NbCL-1].Location_Code, CL[NbCL-1].Starting_Date, CL[NbCL-1].Ending_Date);

			 if ((strlen (CL[NbCL-1].Operator) > 0) && (strcmp (CL[NbCL-1].Operator, "-")))
				{
if (DEBUG) printf ("\n\tProcessing Operator");fflush (stdout);
				 if (	!strcmp (ComOperator.Network_Code, 	CL[NbCL-1].Network_Code) &&
					!strcmp (ComOperator.Site_Code, 	CL[NbCL-1].Site_Code) &&
					!strcmp (ComOperator.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code) &&
					!strcmp (ComOperator.Location_Code, 	CL[NbCL-1].Location_Code))
						{
if (DEBUG) printf ("\n\t\tFound Same Operator (%s)", CL[NbCL-1].Operator);fflush (stdout);
						 if (	!strcmp (ComOperator.Comments, CL[NbCL-1].Operator) &&
							!strcmp (ComOperator.Ending_Date, CL[NbCL-1].Starting_Date))
							{
							 strcpy (ComOperator.Ending_Date, CL[NbCL-1].Ending_Date); 
							}
						 else
							{
                                         		 fprintf (frem, "%s %s %s %s %s %s 'Operator: %s'\n",
                                                  		 ComOperator.Network_Code, ComOperator.Site_Code, ComOperator.SEED_Channel_Code,
                                                  		 ComOperator.Location_Code, ComOperator.Starting_Date, ComOperator.Ending_Date,
                                                  		 GetOperator (ComOperator.Comments));

                                         		 strcpy (ComOperator.Network_Code,      CL[NbCL-1].Network_Code);
                                         		 strcpy (ComOperator.Site_Code,         CL[NbCL-1].Site_Code);
                                         		 strcpy (ComOperator.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                         		 strcpy (ComOperator.Location_Code,     CL[NbCL-1].Location_Code);
                                         		 strcpy (ComOperator.Starting_Date,     CL[NbCL-1].Starting_Date);
                                         		 strcpy (ComOperator.Ending_Date,       CL[NbCL-1].Ending_Date);
                                         		 strcpy (ComOperator.Comments,          CL[NbCL-1].Operator);
							}
						}
				 else
					{
if (DEBUG) printf ("\n\t\tFound Different Operator (%s)", CL[NbCL-1].Operator);fflush (stdout);
					 if (FlagOperator == 1)
                                         	fprintf (frem, "%s %s %s %s %s %s 'Operator: %s'\n",
                                          	  	ComOperator.Network_Code, ComOperator.Site_Code, ComOperator.SEED_Channel_Code,
                                          	  	ComOperator.Location_Code, ComOperator.Starting_Date, ComOperator.Ending_Date,
                                          	  	GetOperator (ComOperator.Comments));
					 else	FlagOperator = 1;

					 strcpy (ComOperator.Network_Code, 	CL[NbCL-1].Network_Code);
					 strcpy (ComOperator.Site_Code, 	CL[NbCL-1].Site_Code);
					 strcpy (ComOperator.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
					 strcpy (ComOperator.Location_Code,     CL[NbCL-1].Location_Code);
					 strcpy (ComOperator.Starting_Date, 	CL[NbCL-1].Starting_Date);
					 strcpy (ComOperator.Ending_Date, 	CL[NbCL-1].Ending_Date);
					 strcpy (ComOperator.Comments, 		CL[NbCL-1].Operator);
					}
				}

			 if ((strlen (CL[NbCL-1].FM_Frequency_Channel) > 0) && (strcmp (CL[NbCL-1].FM_Frequency_Channel, "-")))
                                {
if (DEBUG) printf ("\n\tProcessing Frequency Channel");fflush (stdout);
                                 if (   !strcmp (ComFM.Network_Code,      CL[NbCL-1].Network_Code) &&
                                        !strcmp (ComFM.Site_Code,         CL[NbCL-1].Site_Code) &&
                                        !strcmp (ComFM.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code) &&
                                        !strcmp (ComFM.Location_Code,     CL[NbCL-1].Location_Code))
                                                {
if (DEBUG) printf ("\n\t\tFound Same Channel (%s)", CL[NbCL-1].FM_Frequency_Channel);fflush (stdout);
                                                 if (   !strcmp (ComFM.Comments, CL[NbCL-1].FM_Frequency_Channel) &&
                                                        !strcmp (ComFM.Ending_Date, CL[NbCL-1].Starting_Date))
                                                        {
                                                         strcpy (ComFM.Ending_Date, CL[NbCL-1].Ending_Date); 
                                                        }
                                                 else
                                                        {
                                 			 fprintf (frem, "%s %s %s %s %s %s 'FM Frequency Channel: %s'\n",
                                          			 ComFM.Network_Code, ComFM.Site_Code, ComFM.SEED_Channel_Code,
                                          			 ComFM.Location_Code, ComFM.Starting_Date, ComFM.Ending_Date,
                                          			 ComFM.Comments);

                                                         strcpy (ComFM.Network_Code,      CL[NbCL-1].Network_Code);
                                                         strcpy (ComFM.Site_Code,         CL[NbCL-1].Site_Code);
                                                         strcpy (ComFM.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                                         strcpy (ComFM.Location_Code,     CL[NbCL-1].Location_Code);
                                                         strcpy (ComFM.Starting_Date,     CL[NbCL-1].Starting_Date);
                                                         strcpy (ComFM.Ending_Date,       CL[NbCL-1].Ending_Date);
                                                         strcpy (ComFM.Comments,          CL[NbCL-1].FM_Frequency_Channel);
                                                        }
                                                }
                                 else
                                        {
if (DEBUG) printf ("\n\t\tFound Different Channel (%s)", CL[NbCL-1].FM_Frequency_Channel);fflush (stdout);
                                         if (FlagFM == 1)
						fprintf (frem, "%s %s %s %s %s %s 'FM Frequency Channel: %s'\n",
							 ComFM.Network_Code, ComFM.Site_Code, ComFM.SEED_Channel_Code,
							 ComFM.Location_Code, ComFM.Starting_Date, ComFM.Ending_Date,
							 ComFM.Comments);
                                         else   FlagFM = 1;

                                         strcpy (ComFM.Network_Code,      CL[NbCL-1].Network_Code);
                                         strcpy (ComFM.Site_Code,         CL[NbCL-1].Site_Code);
                                         strcpy (ComFM.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                         strcpy (ComFM.Location_Code,     CL[NbCL-1].Location_Code);
                                         strcpy (ComFM.Starting_Date,     CL[NbCL-1].Starting_Date);
                                         strcpy (ComFM.Ending_Date,       CL[NbCL-1].Ending_Date);
                                         strcpy (ComFM.Comments,          CL[NbCL-1].FM_Frequency_Channel);
                                        }
                                }

			 if ((strlen (CL[NbCL-1].Tape_Track) > 0) && (strcmp (CL[NbCL-1].Tape_Track, "-")))
                                {
if (DEBUG) printf ("\n\tProcessing Tape Track");fflush (stdout);
                                 if (   !strcmp (ComTrack.Network_Code,      CL[NbCL-1].Network_Code) &&
                                        !strcmp (ComTrack.Site_Code,         CL[NbCL-1].Site_Code) &&
                                        !strcmp (ComTrack.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code) &&
                                        !strcmp (ComTrack.Location_Code,     CL[NbCL-1].Location_Code))
                                                {
if (DEBUG) printf ("\n\t\tFound Same Track (%s)", CL[NbCL-1].Tape_Track);fflush (stdout);
                                                 if (   !strcmp (ComTrack.Comments, CL[NbCL-1].Tape_Track) &&
                                                        !strcmp (ComTrack.Ending_Date, CL[NbCL-1].Starting_Date))
                                                        {
                                                         strcpy (ComTrack.Ending_Date, CL[NbCL-1].Ending_Date); 
                                                        }
                                                 else
                                                        {
							 fprintf (frem, "%s %s %s %s %s %s 'Tape Track: %s'\n",
                                          			 ComTrack.Network_Code, ComTrack.Site_Code, ComTrack.SEED_Channel_Code,
                                          			 ComTrack.Location_Code, ComTrack.Starting_Date, ComTrack.Ending_Date,
                                          			 ComTrack.Comments);

                                                         strcpy (ComTrack.Network_Code,      CL[NbCL-1].Network_Code);
                                                         strcpy (ComTrack.Site_Code,         CL[NbCL-1].Site_Code);
                                                         strcpy (ComTrack.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                                         strcpy (ComTrack.Location_Code,     CL[NbCL-1].Location_Code);
                                                         strcpy (ComTrack.Starting_Date,     CL[NbCL-1].Starting_Date);
                                                         strcpy (ComTrack.Ending_Date,       CL[NbCL-1].Ending_Date);
                                                         strcpy (ComTrack.Comments,          CL[NbCL-1].Tape_Track);
                                                        }
                                                }
                                 else
                                        {
if (DEBUG) printf ("\n\t\tFound Different Track (%s)", CL[NbCL-1].Tape_Track);fflush (stdout);
                                         if (FlagTrack == 1)
                                         	fprintf (frem, "%s %s %s %s %s %s 'Tape Track: %s'\n",
                                         		ComTrack.Network_Code, ComTrack.Site_Code, ComTrack.SEED_Channel_Code,
                                         		ComTrack.Location_Code, ComTrack.Starting_Date, ComTrack.Ending_Date,
                                         		ComTrack.Comments);

                                         else   FlagTrack = 1;

                                         strcpy (ComTrack.Network_Code,      CL[NbCL-1].Network_Code);
                                         strcpy (ComTrack.Site_Code,         CL[NbCL-1].Site_Code);
                                         strcpy (ComTrack.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                         strcpy (ComTrack.Location_Code,     CL[NbCL-1].Location_Code);
                                         strcpy (ComTrack.Starting_Date,     CL[NbCL-1].Starting_Date);
                                         strcpy (ComTrack.Ending_Date,       CL[NbCL-1].Ending_Date);
                                         strcpy (ComTrack.Comments,          CL[NbCL-1].Tape_Track);
                                        }
                                }

			 if ((strlen (CL[NbCL-1].Tape_Recorder) > 0) && (strcmp (CL[NbCL-1].Tape_Recorder, "-")))
                                {
if (DEBUG) printf ("\n\tProcessing Tape Recorder");fflush (stdout);
                                 if (   !strcmp (ComRecorder.Network_Code,      CL[NbCL-1].Network_Code) &&
                                        !strcmp (ComRecorder.Site_Code,         CL[NbCL-1].Site_Code) &&
                                        !strcmp (ComRecorder.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code) &&
                                        !strcmp (ComRecorder.Location_Code,     CL[NbCL-1].Location_Code))
                                                {
if (DEBUG) printf ("\n\t\tFound Same Recorder (%s)", CL[NbCL-1].Tape_Recorder);fflush (stdout);
                                                 if (   !strcmp (ComRecorder.Comments, CL[NbCL-1].Tape_Recorder) &&
                                                        !strcmp (ComRecorder.Ending_Date, CL[NbCL-1].Starting_Date))
                                                        {
                                                         strcpy (ComRecorder.Ending_Date, CL[NbCL-1].Ending_Date); 
                                                        }
                                                 else
                                                        {
							 fprintf (frem, "%s %s %s %s %s %s 'Tape Recorder: %s'\n",
								 ComRecorder.Network_Code, ComRecorder.Site_Code, ComRecorder.SEED_Channel_Code,
								 ComRecorder.Location_Code, ComRecorder.Starting_Date, ComRecorder.Ending_Date,
                                                                 ComRecorder.Comments);

                                                         strcpy (ComRecorder.Network_Code,      CL[NbCL-1].Network_Code);
                                                         strcpy (ComRecorder.Site_Code,         CL[NbCL-1].Site_Code);
                                                         strcpy (ComRecorder.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                                         strcpy (ComRecorder.Location_Code,     CL[NbCL-1].Location_Code);
                                                         strcpy (ComRecorder.Starting_Date,     CL[NbCL-1].Starting_Date);
                                                         strcpy (ComRecorder.Ending_Date,       CL[NbCL-1].Ending_Date);
                                                         strcpy (ComRecorder.Comments,          CL[NbCL-1].Tape_Recorder);
                                                        }
                                                }
                                 else
                                        {
if (DEBUG) printf ("\n\t\tFound Different Recorder (%s)", CL[NbCL-1].Tape_Recorder);fflush (stdout);
                                         if (FlagRecorder == 1)
						 fprintf (frem, "%s %s %s %s %s %s 'Tape Recorder: %s'\n",
							 ComRecorder.Network_Code, ComRecorder.Site_Code, ComRecorder.SEED_Channel_Code,
							 ComRecorder.Location_Code, ComRecorder.Starting_Date, ComRecorder.Ending_Date,
                                                         ComRecorder.Comments);
                                         else	FlagRecorder = 1;

                                         strcpy (ComRecorder.Network_Code,      CL[NbCL-1].Network_Code);
                                         strcpy (ComRecorder.Site_Code,         CL[NbCL-1].Site_Code);
                                         strcpy (ComRecorder.SEED_Channel_Code, CL[NbCL-1].SEED_Channel_Code);
                                         strcpy (ComRecorder.Location_Code,     CL[NbCL-1].Location_Code);
                                         strcpy (ComRecorder.Starting_Date,     CL[NbCL-1].Starting_Date);
                                         strcpy (ComRecorder.Ending_Date,       CL[NbCL-1].Ending_Date);
                                         strcpy (ComRecorder.Comments,          CL[NbCL-1].Tape_Recorder);
                                        }
                                }

if (DEBUG) printf ("\n\tProcessing Field Remarks (%s)", CL[NbCL-1].Field_Remarks);fflush (stdout);
			 if ((strlen (CL[NbCL-1].Field_Remarks) > 0) && (strcmp (CL[NbCL-1].Field_Remarks, "-")))
			 	for (z=0;z <= strlen (CL[NbCL-1].Field_Remarks) - 1;z++)
					 fprintf (frem, "%s %s %s %s %s %s '%s'\n",
						  CL[NbCL-1].Network_Code, CL[NbCL-1].Site_Code, CL[NbCL-1].SEED_Channel_Code,
						  CL[NbCL-1].Location_Code, CL[NbCL-1].Starting_Date, CL[NbCL-1].Starting_Date,
						  GetTechRemark (CL[NbCL-1].Field_Remarks[z]));

if (DEBUG) printf ("\n\tProcessing Comments (%s)", CL[NbCL-1].Comments);fflush (stdout);
			 if ((strlen (CL[NbCL-1].Comments) > 0) && (strcmp (CL[NbCL-1].Comments, "-")))
				fprintf (frem, "%s %s %s %s %s %s '%s'\n",
					 CL[NbCL-1].Network_Code, CL[NbCL-1].Site_Code, CL[NbCL-1].SEED_Channel_Code,
					 CL[NbCL-1].Location_Code, CL[NbCL-1].Starting_Date, CL[NbCL-1].Ending_Date,
					 CL[NbCL-1].Comments);
			}

		 reset_token (&tokenc, &tokenv, 0);

		 /* Reading next line */
		 fgets (line, 1023, ftmp);
		}

	 if (FlagOperator == 1)
		fprintf (frem, "%s %s %s %s %s %s 'Operator: %s'\n",
			ComOperator.Network_Code, ComOperator.Site_Code, ComOperator.SEED_Channel_Code,
			ComOperator.Location_Code, ComOperator.Starting_Date, ComOperator.Ending_Date,
			GetOperator (ComOperator.Comments));

	 if (FlagFM == 1)
		fprintf (frem, "%s %s %s %s %s %s 'FM Frequency Channel: %s'\n",
			ComFM.Network_Code, ComFM.Site_Code, ComFM.SEED_Channel_Code,
			ComFM.Location_Code, ComFM.Starting_Date, ComFM.Ending_Date,
			ComFM.Comments);

	 if (FlagTrack == 1)
		fprintf (frem, "%s %s %s %s %s %s 'Tape Track: %s'\n",
			ComTrack.Network_Code, ComTrack.Site_Code, ComTrack.SEED_Channel_Code,
			ComTrack.Location_Code, ComTrack.Starting_Date, ComTrack.Ending_Date,
			ComTrack.Comments);

	 if (FlagRecorder == 1) 
		 fprintf (frem, "%s %s %s %s %s %s 'Tape Recorder: %s'\n",
			 ComRecorder.Network_Code, ComRecorder.Site_Code, ComRecorder.SEED_Channel_Code,
			 ComRecorder.Location_Code, ComRecorder.Starting_Date, ComRecorder.Ending_Date,
                         ComRecorder.Comments);

	 printf (" --> %2d valid channel(s). ", NbCL);

	 /* Any valid channels? */
	 if (NbCL > 0)
		{
		   /**********************************/
		  /* Coalescing channels time spans */
		 /**********************************/
		 NbCL2 = 1;
		 strcpy (CL2[0].Network_Code, 			CL[0].Network_Code);
		 strcpy (CL2[0].Site_Code, 			CL[0].Site_Code);
		 strcpy (CL2[0].Location_Code, 			CL[0].Location_Code);
		 strcpy (CL2[0].USGS_Channel_Code, 		CL[0].USGS_Channel_Code);
		 strcpy (CL2[0].Starting_Date, 			CL[0].Starting_Date);
		 strcpy (CL2[0].Ending_Date, 			CL[0].Ending_Date);
		 strcpy (CL2[0].Latitude, 			CL[0].Latitude);
		 strcpy (CL2[0].Longitude, 			CL[0].Longitude);
		 strcpy (CL2[0].Elevation, 			CL[0].Elevation);
		 strcpy (CL2[0].Depth, 				CL[0].Depth);
		 strcpy (CL2[0].Azimuth, 			CL[0].Azimuth);
		 strcpy (CL2[0].Dip, 				CL[0].Dip);
		 strcpy (CL2[0].Station_Name, 			CL[0].Station_Name);
		 strcpy (CL2[0].Channel_Type, 			CL[0].Channel_Type);
		 strcpy (CL2[0].Sample_Rate, 			CL[0].Sample_Rate);
		 strcpy (CL2[0].Sensor_Serial_Number, 		CL[0].Sensor_Serial_Number);
		 strcpy (CL2[0].Pre_Amp_Serial_Number, 		CL[0].Pre_Amp_Serial_Number);
		 strcpy (CL2[0].Datalogger_Serial_Number, 	CL[0].Datalogger_Serial_Number);
		 strcpy (CL2[0].Component_Number, 		CL[0].Component_Number);
		 strcpy (CL2[0].Pre_Amp_Channel_Number, 	CL[0].Pre_Amp_Channel_Number);
		 strcpy (CL2[0].Primary_Pin_Number, 		CL[0].Primary_Pin_Number);
		 strcpy (CL2[0].FIR_Filter, 			CL[0].FIR_Filter);
		 strcpy (CL2[0].Sensor_Type, 			CL[0].Sensor_Type);
		 strcpy (CL2[0].Sensor_Units, 			CL[0].Sensor_Units);
		 strcpy (CL2[0].Sensor_Sensitivity, 		CL[0].Sensor_Sensitivity);
		 strcpy (CL2[0].Pre_Amp_Type, 			CL[0].Pre_Amp_Type);
		 strcpy (CL2[0].Pre_Amp_Gain, 			CL[0].Pre_Amp_Gain);
		 strcpy (CL2[0].Primary_Datalogger, 		CL[0].Primary_Datalogger);
		 strcpy (CL2[0].Digitizer_Sensitivity, 		CL[0].Digitizer_Sensitivity);
		 strcpy (CL2[0].Sensor_Response, 		CL[0].Sensor_Response);
		 strcpy (CL2[0].Pre_Amp_Response, 		CL[0].Pre_Amp_Response);
		 strcpy (CL2[0].Digitizer_1_Response, 		CL[0].Digitizer_1_Response);
		 strcpy (CL2[0].Location_Reference, 		CL[0].Location_Reference);
		 strcpy (CL2[0].SEED_Channel_Code, 		CL[0].SEED_Channel_Code);
		 strcpy (CL2[0].Decimation_Filter, 		CL[0].Decimation_Filter);
		 strcpy (CL2[0].Decimation_Rate, 		CL[0].Decimation_Rate);
		 strcpy (CL2[0].Decimation_Channel, 		CL[0].Decimation_Channel);

		 for (j=1;j<NbCL;j++)
			{
			 /* Is it the same information? */
			 if ((!strcmp (CL2[NbCL2 - 1].Network_Code,		CL[j].Network_Code))			&&
			    (!strcmp (CL2[NbCL2 - 1].Site_Code,			CL[j].Site_Code))			&&
			    (!strcmp (CL2[NbCL2 - 1].SEED_Channel_Code,		CL[j].SEED_Channel_Code))		&&
			    (!strcmp (CL2[NbCL2 - 1].Location_Code,		CL[j].Location_Code))			&&
			    (!strcmp (CL2[NbCL2 - 1].Latitude,			CL[j].Latitude))			&&
			    (!strcmp (CL2[NbCL2 - 1].Longitude,			CL[j].Longitude))			&&
			    (!strcmp (CL2[NbCL2 - 1].Elevation,			CL[j].Elevation))			&&
			    (!strcmp (CL2[NbCL2 - 1].Depth,			CL[j].Depth))				&&
			    (!strcmp (CL2[NbCL2 - 1].Azimuth,			CL[j].Azimuth))				&&
			    (!strcmp (CL2[NbCL2 - 1].Dip,			CL[j].Dip))				&&
			    (!strcmp (CL2[NbCL2 - 1].Station_Name,		CL[j].Station_Name))			&&
			    (!strcmp (CL2[NbCL2 - 1].Channel_Type,		CL[j].Channel_Type))			&&
			    (!strcmp (CL2[NbCL2 - 1].Sample_Rate,		CL[j].Sample_Rate))			&&
			    (!strcmp (CL2[NbCL2 - 1].Sensor_Serial_Number,	CL[j].Sensor_Serial_Number))		&&
			    (!strcmp (CL2[NbCL2 - 1].Pre_Amp_Serial_Number,	CL[j].Pre_Amp_Serial_Number))		&&
			    (!strcmp (CL2[NbCL2 - 1].Datalogger_Serial_Number,	CL[j].Datalogger_Serial_Number))	&&
			    (!strcmp (CL2[NbCL2 - 1].Component_Number,		CL[j].Component_Number))		&&
			    (!strcmp (CL2[NbCL2 - 1].Pre_Amp_Channel_Number,	CL[j].Pre_Amp_Channel_Number))		&&
			    (!strcmp (CL2[NbCL2 - 1].Primary_Pin_Number,	CL[j].Primary_Pin_Number))		&&
			    (!strcmp (CL2[NbCL2 - 1].FIR_Filter,		CL[j].FIR_Filter))			&&
			    (!strcmp (CL2[NbCL2 - 1].Sensor_Type,		CL[j].Sensor_Type))			&&
			    (!strcmp (CL2[NbCL2 - 1].Sensor_Units,		CL[j].Sensor_Units))			&&
			    (!strcmp (CL2[NbCL2 - 1].Sensor_Sensitivity,	CL[j].Sensor_Sensitivity))		&&
			    (!strcmp (CL2[NbCL2 - 1].Pre_Amp_Type,		CL[j].Pre_Amp_Type))			&&
			    (!strcmp (CL2[NbCL2 - 1].Pre_Amp_Gain,		CL[j].Pre_Amp_Gain))			&&
			    (!strcmp (CL2[NbCL2 - 1].Primary_Datalogger,	CL[j].Primary_Datalogger))		&&
			    (!strcmp (CL2[NbCL2 - 1].Digitizer_Sensitivity,	CL[j].Digitizer_Sensitivity))		&&
			    (!strcmp (CL2[NbCL2 - 1].Sensor_Response,		CL[j].Sensor_Response))			&&
			    (!strcmp (CL2[NbCL2 - 1].Pre_Amp_Response,		CL[j].Pre_Amp_Response))		&&
			    (!strcmp (CL2[NbCL2 - 1].Digitizer_1_Response,	CL[j].Digitizer_1_Response))		&&
			    (!strcmp (CL2[NbCL2 - 1].Location_Reference,	CL[j].Location_Reference))		&&
			    (!strcmp (CL2[NbCL2 - 1].Decimation_Filter,		CL[j].Decimation_Filter))		&&
			    (!strcmp (CL2[NbCL2 - 1].Decimation_Rate,		CL[j].Decimation_Rate))			&&
			    (!strcmp (CL2[NbCL2 - 1].Decimation_Channel,	CL[j].Decimation_Channel))		&&
			    (!strcmp (CL2[NbCL2 - 1].Ending_Date,		CL[j].Starting_Date)))
				{
				 strcpy (CL2[NbCL2 - 1].Ending_Date, CL[j].Ending_Date);
				}
			 else
				{
				 strcpy (CL2[NbCL2].Network_Code, 		CL[j].Network_Code);
				 strcpy (CL2[NbCL2].Site_Code, 			CL[j].Site_Code);
				 strcpy (CL2[NbCL2].Location_Code, 		CL[j].Location_Code);
				 strcpy (CL2[NbCL2].USGS_Channel_Code, 		CL[j].USGS_Channel_Code);
				 strcpy (CL2[NbCL2].Starting_Date, 		CL[j].Starting_Date);
				 strcpy (CL2[NbCL2].Ending_Date, 		CL[j].Ending_Date);
				 strcpy (CL2[NbCL2].Latitude, 			CL[j].Latitude);
				 strcpy (CL2[NbCL2].Longitude, 			CL[j].Longitude);
				 strcpy (CL2[NbCL2].Elevation, 			CL[j].Elevation);
				 strcpy (CL2[NbCL2].Depth, 			CL[j].Depth);
				 strcpy (CL2[NbCL2].Azimuth, 			CL[j].Azimuth);
				 strcpy (CL2[NbCL2].Dip, 			CL[j].Dip);
				 strcpy (CL2[NbCL2].Station_Name, 		CL[j].Station_Name);
				 strcpy (CL2[NbCL2].Channel_Type, 		CL[j].Channel_Type);
				 strcpy (CL2[NbCL2].Sample_Rate, 		CL[j].Sample_Rate);
				 strcpy (CL2[NbCL2].Sensor_Serial_Number, 	CL[j].Sensor_Serial_Number);
				 strcpy (CL2[NbCL2].Pre_Amp_Serial_Number, 	CL[j].Pre_Amp_Serial_Number);
				 strcpy (CL2[NbCL2].Datalogger_Serial_Number, 	CL[j].Datalogger_Serial_Number);
				 strcpy (CL2[NbCL2].Component_Number, 		CL[j].Component_Number);
				 strcpy (CL2[NbCL2].Pre_Amp_Channel_Number, 	CL[j].Pre_Amp_Channel_Number);
				 strcpy (CL2[NbCL2].Primary_Pin_Number, 	CL[j].Primary_Pin_Number);
				 strcpy (CL2[NbCL2].FIR_Filter, 		CL[j].FIR_Filter);
				 strcpy (CL2[NbCL2].Sensor_Type, 		CL[j].Sensor_Type);
				 strcpy (CL2[NbCL2].Sensor_Units, 		CL[j].Sensor_Units);
				 strcpy (CL2[NbCL2].Sensor_Sensitivity, 	CL[j].Sensor_Sensitivity);
				 strcpy (CL2[NbCL2].Pre_Amp_Type, 		CL[j].Pre_Amp_Type);
				 strcpy (CL2[NbCL2].Pre_Amp_Gain, 		CL[j].Pre_Amp_Gain);
				 strcpy (CL2[NbCL2].Primary_Datalogger, 	CL[j].Primary_Datalogger);
				 strcpy (CL2[NbCL2].Digitizer_Sensitivity, 	CL[j].Digitizer_Sensitivity);
				 strcpy (CL2[NbCL2].Sensor_Response, 		CL[j].Sensor_Response);
				 strcpy (CL2[NbCL2].Pre_Amp_Response, 		CL[j].Pre_Amp_Response);
				 strcpy (CL2[NbCL2].Digitizer_1_Response, 	CL[j].Digitizer_1_Response);
				 strcpy (CL2[NbCL2].Location_Reference, 	CL[j].Location_Reference);
				 strcpy (CL2[NbCL2].SEED_Channel_Code, 		CL[j].SEED_Channel_Code);
				 strcpy (CL2[NbCL2].Decimation_Filter, 		CL[j].Decimation_Filter);
				 strcpy (CL2[NbCL2].Decimation_Rate, 		CL[j].Decimation_Rate);
				 strcpy (CL2[NbCL2].Decimation_Channel, 	CL[j].Decimation_Channel);
				 NbCL2++;
				}
			}

	 printf (" --> %2d unique valid channel(s).\n", NbCL2);

		   /*****************************/
		  /* Building Time Spans Array */
		 /*****************************/

		 /* Allocating memory */
		 timespantmp = (char*)malloc (NbCL2*2*14);

		 /* Building Array */
		 for (j=0;j<NbCL2;j++)
			{
			 strcpy (timespantmp + (j*28)     , CL2[j].Starting_Date);
			 strcpy (timespantmp + ((j*28)+14), CL2[j].Ending_Date);
			}

		 /* Sorting Array */
		 qsort ((char *) timespantmp, NbCL2*2, 14, stringcompare);

		   /************************************/
		  /* Building unique time spans array */
		 /************************************/
		 NbTS = 0;
		 strcpy (ts, "");

		 /* Allocating memory */
		 timespan = (char*)malloc (NbCL2*2*14);

		 /* Building Array */
		 for (k=0;k<2*NbCL2;k++)
			{
			 if (strcmp (ts, timespantmp + (k*14)))
				{
				 strcpy (timespan + (NbTS*14), timespantmp + (k*14));
				 NbTS++;
				 strcpy (ts, timespantmp + (k*14));
				}
			}

		 /* Display time spans */
		 for (k=0;k<NbTS;k++)
			printf ("%s\t", timespan + (k*14));


		   /****************************/
		  /* Building sensor.tbl file */
		 /****************************/
		 NbSnsr = 0;

		 /* Browsing channels */
		 for (l=0;l<NbCL2;l++)
			{
			 /* Which Sensor? */
			 Flag_Snsr = -1;
			 for (z=0;z < NbSnsr;z++)
				if (!strcmp (CL2[l].Sensor_Serial_Number, snsrHT[z].set_sn))
					{
					 Flag_Snsr = z;
					 break;
					}

			 /* New Sensor */
			 if (Flag_Snsr == (-1))
				{
				 Flag_Snsr = NbSnsr++;

				 /* Sensor Identification */
				 /*sprintf (snsrHT[Flag_Snsr].set_sid, "NC%s", CL2[l].Sensor_Type);*/

				 /* Replacing blanks */
				 for (zz=0;zz<strlen (snsrHT[Flag_Snsr].set_sid);zz++)
					if (snsrHT[Flag_Snsr].set_sid[zz] == ' ')
						snsrHT[Flag_Snsr].set_sid[zz] = '_';

				 /* Sensor Serial Number */
				 strcpy (snsrHT[Flag_Snsr].set_sn, CL2[l].Sensor_Serial_Number);

				 /* Sensor Starting & Ending Dates */
				 strcpy (snsrHT[Flag_Snsr].set_sdat, CL2[l].Starting_Date);
				 strcpy (snsrHT[Flag_Snsr].set_edat, CL2[l].Ending_Date);

				 /* Components Information */
				 sprintf (nresp, "%s/%s", HARD_PATH, CL2[l].Sensor_Response);
				 if ((fresp = fopen (nresp, "rt")) == NULL)
					{
					 fprintf (ferrortbl, "\nError: Couldn't open file %s.\n", nresp);
					 exit (-18);
					}

				 fgets (tmpline2, 254, fresp);

				 /* SENSOR line */
				 sscanf (tmpline2, "%*s %s %*s %*s %*s %d", snsrHT[Flag_Snsr].set_sid, &snsrHT[Flag_Snsr].set_ncmp);

				 for (z=0;z < snsrHT[Flag_Snsr].set_ncmp; z++)
					{
					 snsrHT[Flag_Snsr].set_comp[z].set_cmpnb = z + 1;
					 fgets (tmpline2, 254, fresp);

					 /* SC line */
					 sscanf (tmpline2, "%*s %c %c %s %lf %lf %d",
							&snsrHT[Flag_Snsr].set_comp[z].set_chan,
							&snsrHT[Flag_Snsr].set_comp[z].set_styp,
							snsrHT[Flag_Snsr].set_comp[z].set_units,
							&snsrHT[Flag_Snsr].set_comp[z].set_sens,
							&snsrHT[Flag_Snsr].set_comp[z].set_freq,
							&snsrHT[Flag_Snsr].set_comp[z].set_nlin);

					 strcpy (snsrHT[Flag_Snsr].set_comp[z].set_resp, "");
					 for (zz=0; zz < snsrHT[Flag_Snsr].set_comp[z].set_nlin; zz++)
						{
						 fgets (tmpline2, 254, fresp);

						 /* Response line */
						 strcat (snsrHT[Flag_Snsr].set_comp[z].set_resp, tmpline2);
						}
					}

				 fclose (fresp);
				}
			 else
				{
				 /* Updating time span */
				 if (strcmp (snsrHT[Flag_Snsr].set_sdat, CL2[l].Starting_Date) > 0)
					strcpy (snsrHT[Flag_Snsr].set_sdat, CL2[l].Starting_Date);

				 if (strcmp (snsrHT[Flag_Snsr].set_edat, CL2[l].Ending_Date) < 0)
					strcpy (snsrHT[Flag_Snsr].set_edat, CL2[l].Ending_Date);
				}

			 /* Component Sensitivity */
			 CmpnntNb = (-1);
			 for (m=0;m<snsrHT[Flag_Snsr].set_ncmp;m++)
				{
				 if (snsrHT[Flag_Snsr].set_comp[m].set_cmpnb == atoi (CL2[l].Component_Number))
					{
					 CmpnntNb = m;
					 break;
					}
				}

			 if (CmpnntNb == (-1))
				{
				 fprintf (ferrortbl, "\nError [%s.%s.%s]: Unknown Sensor Component.", CL2[l].Network_Code, CL2[l].Site_Code, CL2[l].SEED_Channel_Code);
				 exit (-19);
				}
			 else
				{
				 /* Component Sensitivity */
				 if (atof (CL2[l].Sensor_Sensitivity) != 0.)
					snsrHT[Flag_Snsr].set_comp[CmpnntNb].set_sens = atof (CL2[l].Sensor_Sensitivity);
				}
			}

		 /* Writing Sensors Information */
		 for (z=0;z<NbSnsr;z++)
			{
			 fprintf (fsensor, "\nSENSOR\t%s\t%s\t%s\t%s\t%d\n",
				   snsrHT[z].set_sid, snsrHT[z].set_sn, snsrHT[z].set_sdat, snsrHT[z].set_edat, snsrHT[z].set_ncmp);

			 /* Writing SC lines */
			 for (l=0;l<snsrHT[z].set_ncmp;l++)
				{
				 /* Writing SC and response lines */
				 fprintf (fsensor, "SC\t%c\t%c\t%s\t%f\t%f\t%d\n%s",
			 	  	  snsrHT[z].set_comp[l].set_chan, snsrHT[z].set_comp[l].set_styp,
					  snsrHT[z].set_comp[l].set_units, snsrHT[z].set_comp[l].set_sens,
					  snsrHT[z].set_comp[l].set_freq, snsrHT[z].set_comp[l].set_nlin,
					  snsrHT[z].set_comp[l].set_resp);
				}

			 fprintf (fsensor, "\n");
			}


		   /****************************/
		  /* Building preamp.tbl file */
		 /****************************/
		 NbPrmp = 0;

		 /* Browsing channels */
		 for (l=0;l<NbCL2;l++)
			{
			 /* Is there a Preamp? */
			 if (!strcmp (CL2[l].Pre_Amp_Serial_Number, ""))
				continue;

			 /* Which Preamp? */
			 Flag_Prmp = -1;
			 for (z=0;z < NbPrmp;z++)
				if (!strcmp (CL2[l].Pre_Amp_Serial_Number, prmpHT[z].pat_sn))
					{
					 Flag_Prmp = z;
					 break;
					}

			 /* New Preamp */
			 if (Flag_Prmp == (-1))
				{
				 Flag_Prmp = NbPrmp++;

				 /* Preamp Identification */
				 /*sprintf (prmpHT[Flag_Prmp].pat_paid, "NC%s", CL2[l].Pre_Amp_Type);*/

				 /* Replacing blanks */
				 for (zz=0;zz<strlen (prmpHT[Flag_Prmp].pat_paid);zz++)
					if (prmpHT[Flag_Prmp].pat_paid[zz] == ' ')
						prmpHT[Flag_Prmp].pat_paid[zz] = '_';

				 /* Preamp Serial Number */
				 strcpy (prmpHT[Flag_Prmp].pat_sn, CL2[l].Pre_Amp_Serial_Number);

				 /* Preamp Starting & Ending Dates */
				 strcpy (prmpHT[Flag_Prmp].pat_sdat, CL2[l].Starting_Date);
				 strcpy (prmpHT[Flag_Prmp].pat_edat, CL2[l].Ending_Date);

				 /* Channels Information */
				 sprintf (nresp, "%s/%s", HARD_PATH, CL2[l].Pre_Amp_Response);
				 if ((fresp = fopen (nresp, "rt")) == NULL)
					{
					 fprintf (ferrortbl, "\nError: Couldn't open file %s.\n", nresp);
					 exit (-20);
					}

				 fgets (tmpline2, 254, fresp);

				 /* PREAMP line */
				 sscanf (tmpline2, "%*s %s %*s %*s %*s %d", prmpHT[Flag_Prmp].pat_paid, &prmpHT[Flag_Prmp].pat_nchn);

				 for (z=0;z < prmpHT[Flag_Prmp].pat_nchn; z++)
					{
					 prmpHT[Flag_Prmp].pat_chn[z].pat_ngain = 1;
					 prmpHT[Flag_Prmp].pat_chn[z].pat_gain[0] = 0.;

					 fgets (tmpline2, 254, fresp);

					 /* PA line */
					 sscanf (tmpline2, "%*s %d %lf %lf %d",
							&prmpHT[Flag_Prmp].pat_chn[z].pat_chan,
							&prmpHT[Flag_Prmp].pat_chn[z].pat_gain[1],
							&prmpHT[Flag_Prmp].pat_chn[z].pat_freq,
							&prmpHT[Flag_Prmp].pat_chn[z].pat_nlin);

					 strcpy (prmpHT[Flag_Prmp].pat_chn[z].pat_resp, "");
					 for (zz=0; zz < prmpHT[Flag_Prmp].pat_chn[z].pat_nlin; zz++)
						{
						 fgets (tmpline2, 254, fresp);

						 /* Response line */
						 strcat (prmpHT[Flag_Prmp].pat_chn[z].pat_resp, tmpline2);
						}
					}

				 fclose (fresp);
				}
			 else
				{
				 /* Updating time span */
				 if (strcmp (prmpHT[Flag_Prmp].pat_sdat, CL2[l].Starting_Date) > 0)
					strcpy (prmpHT[Flag_Prmp].pat_sdat, CL2[l].Starting_Date);

				 if (strcmp (prmpHT[Flag_Prmp].pat_edat, CL2[l].Ending_Date) < 0)
					strcpy (prmpHT[Flag_Prmp].pat_edat, CL2[l].Ending_Date);
				}

			 /* Channel Gain */
			 ChnnlNb = (-1);
			 for (m=0;m<prmpHT[Flag_Prmp].pat_nchn;m++)
				{
				 if (prmpHT[Flag_Prmp].pat_chn[m].pat_chan == atoi (CL2[l].Pre_Amp_Channel_Number))
					{
					 ChnnlNb = m;
					 break;
					}
				}

			 if (ChnnlNb == (-1))
				{
				 fprintf (ferrortbl, "\nError [%s.%s.%s]: Unknown Preamp Channel.", CL2[l].Network_Code, CL2[l].Site_Code, CL2[l].SEED_Channel_Code);
				 exit (-21);
				}
			 else
				{
				 /* Channel Gain */
				 if (atof (CL2[l].Pre_Amp_Gain) != 0.)
					{
					 /* Preamp gain changed? */
					 if (atof (CL2[l].Pre_Amp_Gain) != prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_gain[prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_ngain-1])
						{
						 strcpy (prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_gsdat[prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_ngain], CL2[l].Starting_Date);

						 prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_gain[prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_ngain] = atof (CL2[l].Pre_Amp_Gain);

						 prmpHT[Flag_Prmp].pat_chn[ChnnlNb].pat_ngain++;
						}
					}
				}
			}

		 /* Writing Preamps Information */
		 for (z=0;z<NbPrmp;z++)
			{
			 for (xx=1;xx<prmpHT[z].pat_chn[0].pat_ngain;xx++)
			 	{
			 	 if (xx == prmpHT[z].pat_chn[0].pat_ngain - 1)
				 	fprintf (fpreamp, "PREAMP\t%s\t%s\t%s\t%s\t%d\n",
					  	prmpHT[z].pat_paid, prmpHT[z].pat_sn, prmpHT[z].pat_chn[0].pat_gsdat[xx],
					  	prmpHT[z].pat_edat, prmpHT[z].pat_nchn);
				 else	fprintf (fpreamp, "PREAMP\t%s\t%s\t%s\t%s\t%d\n",
					  	prmpHT[z].pat_paid, prmpHT[z].pat_sn, prmpHT[z].pat_chn[0].pat_gsdat[xx],
					  	prmpHT[z].pat_chn[0].pat_gsdat[xx+1], prmpHT[z].pat_nchn);
				   
				 /* Writing PA lines */
				 for (l=0;l<prmpHT[z].pat_nchn;l++)
					{
					 if (prmpHT[z].pat_chn[l].pat_ngain == 1)
						prmpHT[z].pat_chn[l].pat_ngain = 2;

					 /* Writing PA and response lines */
					 if (xx < prmpHT[z].pat_chn[l].pat_ngain)
					 	 fprintf (fpreamp, "PA\t%d\t%f\t%f\t%d\n%s",
						  prmpHT[z].pat_chn[l].pat_chan, prmpHT[z].pat_chn[l].pat_gain[xx],
						  prmpHT[z].pat_chn[l].pat_freq, prmpHT[z].pat_chn[l].pat_nlin,
						  prmpHT[z].pat_chn[l].pat_resp);
					 else	fprintf (fpreamp, "PA\t%d\t%f\t%f\t%d\n%s",
						  prmpHT[z].pat_chn[l].pat_chan, prmpHT[z].pat_chn[l].pat_gain[prmpHT[z].pat_chn[l].pat_ngain-1],
						  prmpHT[z].pat_chn[l].pat_freq, prmpHT[z].pat_chn[l].pat_nlin,
						  prmpHT[z].pat_chn[l].pat_resp);
					}
					
				 fprintf (fpreamp, "\n");
				}
			}


		   /****************************/
		  /* Building datlog.tbl file */
		 /****************************/

		 /* Browsing time spans */
		 for (k=0;k<NbTS-1;k++)
			{
			 NbData = 0;

			 /* Browsing channels */
			 for (l=0;l<NbCL2;l++)
				{
				 /* Multiple-Site Datalogger? */
				 if (strcmp (CL2[l].Digitizer_1_Response, "^"))
					continue;

				 /* Is channel within the time span? */
				 if ((strcmp (timespan + (k*14), CL2[l].Starting_Date) >= 0) &&
				     (strcmp (timespan + (k*14), CL2[l].Ending_Date)   <  0))
					{
				 /* Multiple-Site Datalogger? */
				 if (strcmp (CL2[l].Digitizer_1_Response, "^"))
					continue;

					 /* Which Datalogger? */
					 Flag_Datlog  = -1;
					 for (z=0;z<NbData;z++)
						if (!strcmp (dtlgr[z].dlt_sn, CL2[l].Datalogger_Serial_Number))
							{
							 Flag_Datlog = z;
							 break;
							}

					 /* New Datalogger */
					 if (Flag_Datlog == (-1))
						{
						 Flag_Datlog = NbData++;

						 /* Number of boards */
						 dtlgr[Flag_Datlog].dlt_ndm = 0;

						 /* Datalogger Type */
						 sprintf (dtlgr[Flag_Datlog].dlt_type, "%s", CL2[l].Primary_Datalogger);

						 /* Replacing blanks */
						 for (zz=0;zz<strlen (dtlgr[Flag_Datlog].dlt_type);zz++)
							if (dtlgr[Flag_Datlog].dlt_type[zz] == ' ')
								dtlgr[Flag_Datlog].dlt_type[zz] = '_';


						 /* Datalogger serial number */
						 strcpy (dtlgr[Flag_Datlog].dlt_sn, CL2[l].Datalogger_Serial_Number);

						 /* Datalogger firmware number */
						 sprintf (dtlgr[Flag_Datlog].dlt_firm, "%s", "NULL");

						 /* Datalogger start date */
						 strcpy (dtlgr[Flag_Datlog].dlt_sdat, timespan + (k*14));

						 /* Datalogger end date */
						 strcpy (dtlgr[Flag_Datlog].dlt_edat, timespan + ((k+1)*14));
						}					 

					 /* Board Information */
					 BoardNb = (-1);
					 for (m=0;m<dtlgr[Flag_Datlog].dlt_ndm;m++)
						{
						 if (!strcmp (dtlgr[Flag_Datlog].b[m].dlt_btype, CL2[l].Channel_Type))
							BoardNb = m;
						}

					 /* New Board? */
					 if (BoardNb == (-1))
						{
						 BoardNb = dtlgr[Flag_Datlog].dlt_ndm;

						 /* Board Type */
						 strcpy (dtlgr[Flag_Datlog].b[BoardNb].dlt_btype, CL2[l].Channel_Type);

						 /* Board Number */
						 dtlgr[Flag_Datlog].b[BoardNb].dlt_slot = BoardNb + 1;

						 /* Board Serial Number */
						 sprintf (dtlgr[Flag_Datlog].b[BoardNb].dlt_dmsn, "%s%.1s", dtlgr[Flag_Datlog].dlt_sn, dtlgr[Flag_Datlog].b[BoardNb].dlt_btype);

						 /* Number of modules */
						 dtlgr[Flag_Datlog].b[BoardNb].dlt_nch = 0;

						 /* Incrementing the number of boards */
						 dtlgr[Flag_Datlog].dlt_ndm++;
						}

					 /* Module Information */
					 ModuleNb = dtlgr[Flag_Datlog].b[BoardNb].dlt_nch;

					 /* Module Number */
					 dtlgr[Flag_Datlog].b[BoardNb].m[ModuleNb].dlt_chan = ModuleNb + 1;

					 /* Module Sensitivity */
					 dtlgr[Flag_Datlog].b[BoardNb].m[ModuleNb].dlt_sens = atof (CL2[l].Digitizer_Sensitivity);

					 /* Incrementing the number of modules */
					 dtlgr[Flag_Datlog].b[BoardNb].dlt_nch++;
					}
				}

			 /* Writing Dataloggers Information */
			 for (z=0;z<NbData;z++)
				{
				 fprintf (fdatlog, "DATLOG\t%s\t%s\t%s\t%s\t%s\t%d\n",
				 	  dtlgr[z].dlt_type, dtlgr[z].dlt_sn, dtlgr[z].dlt_firm, dtlgr[z].dlt_sdat, dtlgr[z].dlt_edat, dtlgr[z].dlt_ndm);

				 /* Writing DM & DMCS lines */
				 for (l=0;l<dtlgr[z].dlt_ndm;l++)
					{
					 /* Writing DM line */
					 fprintf (fdatlog, "DM\t%d\t%s\t%d\n",
			 	  		  dtlgr[z].b[l].dlt_slot, dtlgr[z].b[l].dlt_dmsn, dtlgr[z].b[l].dlt_nch);

					 for (m=0;m<dtlgr[z].b[l].dlt_nch;m++)
						{
						 /* Writing DMCS line */
						 fprintf (fdatlog, "DMCS\t%d\t%11.6lf\n",
			 	  			  dtlgr[z].b[l].m[m].dlt_chan, dtlgr[z].b[l].m[m].dlt_sens);
						}
					}

				 fprintf (fdatlog, "\n");
				}
			}


		   /*************************/
		  /* Building sta.tbl file */
		 /*************************/

		 /* Browsing time spans */
		 for (k=0;k<NbTS-1;k++)
			{
			 int	NbDigi[128];	/* Number of Digitizers */
			 int	NbDigiM[128];	/* Number of Modules (Main Board) */
			 int	NbDigiA[128];	/* Number of Modules (Auxiliary Board) */
			 int	NbDigiE[128];	/* Number of Modules (External Board) */
			 int	NbDigiD[128];	/* Number of Modules (Digital Board) */
			 int	DigiNbM[128];	/* Digitizer (Main Board) Number */
			 int	DigiNbA[128];	/* Digitizer (Auxiliary Board) Number */
			 int	DigiNbE[128];	/* Digitizer (External Board) Number */
			 int	DigiNbD[128];	/* Digitizer (Digital Board) Number */
			 int	DataNb   = -1;	/* Datalogger Number */
			 int	SensorNb = -1;	/* Sensor Number */
			 int	PreAmpNb = -1;	/* Pre-Amplifier Number */
			 int	BoardNb;	/* Board Number */
			 int	ConnNb;		/* Connector Number */
			 char	ChanType[128];	/* Channel Type */
			 int	DigiBoard[128];	/* Digitizer Boards List */


			 stinst.NbSensor = 0;
			 stinst.NbPreAmp = 0;
			 stinst.NbDigi   = 0;
			 stinst.NbData   = 0;
			 strcpy (stinst.SDate, timespan + (k*14));
			 strcpy (stinst.EDate, timespan + ((k+1)*14));
			 stinst.NbChannel = 0;

			 /* Resetting number of duplicate channels */
			 NbDupCha2 = 0;

			 /* Browsing channels */
			 for (l=0;l<NbCL2;l++)
				{
				 /* Is channel within the time span? */
				 if ((strcmp (timespan + (k*14), CL2[l].Starting_Date) >= 0) &&
				     (strcmp (timespan + (k*14), CL2[l].Ending_Date)   <  0))
					{
					 char	dctmp[50];
					 int	FlagDC = 0;

					 /* Testing for Duplicate Channels */
					 sprintf (dctmp, "%s.%s", CL2[l].SEED_Channel_Code, CL2[l].Location_Code);
					 for (z=0;z < NbDupCha2; z++)
						{
						 if (!strcmp (dctmp, DupCha2[z]))
							{
							 FlagDC = 1;
							 break;
							}
						}

					 if (FlagDC == 0)
						{
						 strcpy (DupCha2[NbDupCha2++], dctmp);
						}
					 else
						{
						 fprintf (ferrortbl, "\nError [%s.%s.%s]: Duplicate Channel Codes in .tbl file.", CL2[l].Site_Code, CL2[l].SEED_Channel_Code, CL2[l].Location_Code);
						}

					 /* 1st Channel? */
					 if (Flag_Station == 0)
						{
						 Flag_Station = 1;

						 /* Converting elevation */
						 if (!strcmp (CL2[l].Elevation, ""))
							strcpy (CL2[l].Elevation, "0.0");

						 /* Writing SCODE line */
						 for (z=0;z<strlen (CL2[l].Site_Code);z++)
							CL2[l].Site_Code[z] = toupper (CL2[l].Site_Code[z]);

						 fprintf (fsta, "SCODE\t%s\t%s\t%s\t%s\t%s\t\"%s\"\n",
							  CL2[l].Site_Code, CL2[l].Network_Code, CL2[l].Latitude,
							  CL2[l].Longitude, CL2[l].Elevation, CL2[l].Station_Name);
						}

					 /* Datalogger Number? */
					 DataNb = -1;
					 for (z=1;z<=stinst.NbData;z++)
						if (!strcmp (CL2[l].Datalogger_Serial_Number, stinst.Data[z].DataSN))
							{
							 DataNb = z;
							 break;
							}

					 /* New Datalogger */
					 if (DataNb == (-1))
						{
						 DataNb = ++stinst.NbData;
						 strcpy (stinst.Data[DataNb].DataSN, CL2[l].Datalogger_Serial_Number);
						 stinst.Data[DataNb].NbChannel = 0;
						 strcpy (stinst.Data[DataNb].Data, "");

						 stinst.Data[DataNb].NbDigi = 0;

						 DigiNbM[DataNb]  = -1;
						 DigiNbA[DataNb]  = -1;
						 DigiNbE[DataNb]  = -1;
						 DigiNbD[DataNb]  = -1;
						 NbDigi[DataNb]   =  0;
						 NbDigiM[DataNb]  =  0;
						 NbDigiA[DataNb]  =  0;
						 NbDigiE[DataNb]  =  0;
						 NbDigiD[DataNb]  =  0;
						}

					 stinst.Data[DataNb].NbChannel++;
					 stinst.NbChannel++;

					   /**********************/
					  /* Sensor Information */
					 /**********************/

					 /* Converting Azimuth */
					 if (!strcmp (CL2[l].Dip, ""))
						strcpy (CL2[l].Dip, "0.0");

					 if (!strcmp (CL2[l].Azimuth, ""))
						strcpy (CL2[l].Azimuth, "0.0");

					 /* Converting elevation */
					 if (!strcmp (CL2[l].Elevation, ""))
						strcpy (CL2[l].Elevation, "0.0");

					 /* Converting depth */
					 if (!strcmp (CL2[l].Depth, ""))
						strcpy (CL2[l].Depth, "0.0");

					 /* Sensor Number? */
					 SensorNb = -1;
					 for (z=1;z<=stinst.NbSensor;z++)
						if (!strcmp (CL2[l].Sensor_Serial_Number, stinst.Sensor[z].SensorSN))
							{
							 SensorNb = z;
							 stinst.Sensor[SensorNb].NbComp++;
							 break;
							}

					 /* New Sensor */
					 if (SensorNb == (-1))
						{
						 SensorNb = ++stinst.NbSensor;
						 strcpy (stinst.Sensor[SensorNb].SensorSN, CL2[l].Sensor_Serial_Number);
						 strcpy (stinst.Sensor[SensorNb].Latitude, CL2[l].Latitude);
						 strcpy (stinst.Sensor[SensorNb].Longitude, CL2[l].Longitude);
						 strcpy (stinst.Sensor[SensorNb].Elevation, CL2[l].Elevation);
						 strcpy (stinst.Sensor[SensorNb].Depth, CL2[l].Depth);
						 stinst.Sensor[SensorNb].NbComp = 0;
						 strcpy (stinst.Sensor[SensorNb].Senca, "");
						}

					 /* Converting channel type */
					 if (!strcmp (CL2[l].Channel_Type, "P"))
						{
						 if (DigiNbM[DataNb] == (-1))
							{
							 DigiNbM[DataNb] = ++NbDigi[DataNb];

							 stinst.NbDigi++;
							 stinst.Data[DataNb].NbDigi++;

							 sprintf (stinst.Data[DataNb].Digi[DigiNbM[DataNb]].DigiSN, "%s%s", CL2[l].Datalogger_Serial_Number, CL2[l].Channel_Type);
							 strcpy (stinst.Data[DataNb].Digi[DigiNbM[DataNb]].DIGICA, "");
							 stinst.Data[DataNb].Digi[DigiNbM[DataNb]].NbModule = 0;

							 DigiBoard[stinst.NbDigi] = DigiNbM[DataNb]+(DataNb*10);
							}

						 BoardNb = DigiNbM[DataNb];
						 ConnNb = ++NbDigiM[DataNb];
						 stinst.Data[DataNb].Digi[BoardNb].NbModule++;
						 strcpy (ChanType, "DSP");
						}
					 else if (!strcmp (CL2[l].Channel_Type, "A"))
						{
						 if (DigiNbA[DataNb] == (-1))
							{
							 DigiNbA[DataNb] = ++NbDigi[DataNb];

							 stinst.NbDigi++;
							 stinst.Data[DataNb].NbDigi++;

							 sprintf (stinst.Data[DataNb].Digi[DigiNbA[DataNb]].DigiSN, "%s%s", CL2[l].Datalogger_Serial_Number, CL2[l].Channel_Type);
							 strcpy (stinst.Data[DataNb].Digi[DigiNbA[DataNb]].DIGICA, "");
							 stinst.Data[DataNb].Digi[DigiNbA[DataNb]].NbModule = 0;

							 DigiBoard[stinst.NbDigi] = DigiNbA[DataNb]+(DataNb*10);
							}

						 BoardNb = DigiNbA[DataNb];
						 ConnNb = ++NbDigiA[DataNb];
						 stinst.Data[DataNb].Digi[BoardNb].NbModule++;
						 strcpy (ChanType, "AUX");
						}
					 else if (!strcmp (CL2[l].Channel_Type, "E"))
						{
						 if (DigiNbE[DataNb] == (-1))
							{
							 DigiNbE[DataNb] = ++NbDigi[DataNb];

							 stinst.NbDigi++;
							 stinst.Data[DataNb].NbDigi++;

							 sprintf (stinst.Data[DataNb].Digi[DigiNbE[DataNb]].DigiSN, "%s%s", CL2[l].Datalogger_Serial_Number, CL2[l].Channel_Type);
							 strcpy (stinst.Data[DataNb].Digi[DigiNbE[DataNb]].DIGICA, "");
							 stinst.Data[DataNb].Digi[DigiNbE[DataNb]].NbModule = 0;

							 DigiBoard[stinst.NbDigi] = DigiNbE[DataNb]+(DataNb*10);
							}

						 BoardNb = DigiNbE[DataNb];
						 ConnNb = ++NbDigiE[DataNb];
						 stinst.Data[DataNb].Digi[BoardNb].NbModule++;
						 strcpy (ChanType, "EXT");
						}
					 else if (!strcmp (CL2[l].Channel_Type, "D"))
						{
						 if (DigiNbD[DataNb] == (-1))
							{
							 DigiNbD[DataNb] = ++NbDigi[DataNb];

							 stinst.NbDigi++;
							 stinst.Data[DataNb].NbDigi++;

							 sprintf (stinst.Data[DataNb].Digi[DigiNbD[DataNb]].DigiSN, "%s%s", CL2[l].Datalogger_Serial_Number, CL2[l].Channel_Type);
							 strcpy (stinst.Data[DataNb].Digi[DigiNbD[DataNb]].DIGICA, "");
							 stinst.Data[DataNb].Digi[DigiNbD[DataNb]].NbModule = 0;

							 DigiBoard[stinst.NbDigi] = DigiNbD[DataNb]+(DataNb*10);
							}

						 BoardNb = DigiNbD[DataNb];
						 ConnNb = ++NbDigiD[DataNb];
						 stinst.Data[DataNb].Digi[BoardNb].NbModule++;
						 strcpy (ChanType, "DIG");
						}

					 /* Is there a pre-amplifier? */
					 if (strcmp (CL2[l].Pre_Amp_Serial_Number, ""))
						{
						 /* Pre-Amplifier Number? */
						 PreAmpNb = -1;
						 for (z=1;z<=stinst.NbPreAmp;z++)
							if (!strcmp (CL2[l].Pre_Amp_Serial_Number, stinst.PreAmp[z].PreAmpSN))
								{
								 PreAmpNb = z;
								 stinst.PreAmp[PreAmpNb].NbChan++;
								 break;
								}

						 /* New Pre-Amplifier */
						 if (PreAmpNb == (-1))
							{
							 PreAmpNb = ++stinst.NbPreAmp;
							 strcpy (stinst.PreAmp[PreAmpNb].PreAmpSN, CL2[l].Pre_Amp_Serial_Number);
							 stinst.PreAmp[PreAmpNb].NbChan = 0;
							 strcpy (stinst.PreAmp[PreAmpNb].Pampca, "");
							}

						 /* Writing SENCA line */
						 sprintf (tmpline, "SENCA\t%s\t%d\t%s\t%s\t%s\t%s\t%c\n",
							  CL2[l].Component_Number, PreAmpNb, ChanType, CL2[l].Azimuth,
							  CL2[l].Dip, CL2[l].Pre_Amp_Channel_Number, 'F');

						 strcat (stinst.Sensor[SensorNb].Senca, tmpline);

						   /*****************************/
						  /* Pre-Amplifier Information */
						 /*****************************/

						 /* Writing PAMPCA line */
						 sprintf (tmpline, "PAMPCA\t%d\t?%d?\t%s\t%d\n",
							  PreAmpNb, BoardNb+(DataNb*10), CL2[l].Pre_Amp_Channel_Number, ConnNb);

						 strcat (stinst.PreAmp[PreAmpNb].Pampca, tmpline);
						}
					 else
						{
						 /* Writing SENCA line */
						 sprintf (tmpline, "SENCA\t%s\t?%d?\t%s\t%s\t%s\t%d\t%c\n",
							  CL2[l].Component_Number, BoardNb+(DataNb*10), ChanType, CL2[l].Azimuth,
							  CL2[l].Dip, ConnNb, 'D');

						 strcat (stinst.Sensor[SensorNb].Senca, tmpline);
						}

					   /*************************/
					  /* Digitizer Information */
					 /*************************/

					 /* Writing DIGICA line */
					 /* Multiple-Site Datalogger? */
					 if (strcmp (CL2[l].Digitizer_1_Response, "^"))
						{
						 if (strstr (CL2[l].Datalogger_Serial_Number, "Nanometrics") == NULL)
							pin_tmp = atoi (CL2[l].Primary_Pin_Number) + 1;
						 else	pin_tmp = atoi (CL2[l].Primary_Pin_Number);

						 sprintf (tmpline, "DIGICA\t?%d?\t%d\t%s\t%d\t%d\t%s\t%-d\n",
							  BoardNb+(DataNb*10), DataNb, ChanType, ConnNb,
							  pin_tmp, "N", pin_tmp);
						}
					 else	sprintf (tmpline, "DIGICA\t?%d?\t%d\t%s\t%d\t%s\t%s\t%-d\n",
							  BoardNb+(DataNb*10), DataNb, ChanType, ConnNb,
							  CL2[l].Primary_Pin_Number, "N", ConnNb);

					 strcat (stinst.Data[DataNb].Digi[BoardNb].DIGICA, tmpline);

					   /**************************/
					  /* Datalogger Information */
					 /**************************/

					 /* Number of logical channels */
					 if (atof (CL2[l].Decimation_Rate) != 0.)
						nblines = 2;
					 else	nblines = 1;

					 /* Writing DLOGCA line */
					 if ((strcmp (CL2[l].Digitizer_1_Response, "^")) && (strstr (CL2[l].Datalogger_Serial_Number, "Nanometrics") == NULL))
						pin_tmp = atoi (CL2[l].Primary_Pin_Number) + 1;
					 else	pin_tmp = atoi (CL2[l].Primary_Pin_Number);

					 sprintf (tmpline, "DLOGCA\t%d\t%d\t%s\t?%d?\t%d\t%s\t%d\n",
						  DataNb, pin_tmp, ChanType,
						  BoardNb+(DataNb*10), ConnNb, CL2[l].SEED_Channel_Code+1, nblines);

					 strcat (stinst.Data[DataNb].Data, tmpline);

					 /* Writing DLOGFC lines */
					 /* Location Code */
					 if (	(!strcmp (CL2[l].Location_Code, "  ")) ||
						(!strcmp (CL2[l].Location_Code, "")) ||
						(!strcmp (CL2[l].Location_Code, "--")))
							strcpy (tmploc, "");
					 else	sprintf (tmploc, ".%s", CL2[l].Location_Code);

					 /* FIR Filter */
					 if (	(!strcmp (CL2[l].FIR_Filter, "")) ||
						(!strcmp (CL2[l].FIR_Filter, "-")) ||
						(!strcmp (CL2[l].FIR_Filter, "~")))
							strcpy (tmpfir, "NULL");
					 else	strcpy (tmpfir, CL2[l].FIR_Filter);

					 sprintf (tmpline,
						 "DLOGFC\t%d\t%s%s:%s.%s\t%7.2f\t%7.2f\t%s\t%s\t%s\t%s\t%s\t%s\n",
						  DataNb, CL2[l].SEED_Channel_Code, tmploc, CL2[l].USGS_Channel_Code, "CUSP",
						  atof (CL2[l].Sample_Rate),
						  atof (CL2[l].Sample_Rate) / (4.), "1.d-2",
						  "CG", tmpfir, FORMAT, COMPRESSION, BLOCK_SIZE);

					 strcat (stinst.Data[DataNb].Data, tmpline);

					 /* Decimated Channel? */
					 if (atof (CL2[l].Decimation_Rate) != 0.)
						{
						 sprintf (tmpline,
							 "DLOGFC\t%d\t%s%s\t%7.2f\t%7.2f\t%s\t%s\t%s\t%s\t%s\t%s\n",
							  DataNb, CL2[l].Decimation_Channel, tmploc,
							  atof (CL2[l].Decimation_Rate),
							  atof (CL2[l].Decimation_Rate) / (4.), "1.d-2",
							  "CG", CL2[l].Decimation_Filter, FORMAT, COMPRESSION, BLOCK_SIZE);

						 strcat (stinst.Data[DataNb].Data, tmpline);
						}
					}
				}

			   /***********************/
			  /* Writing Information */
			 /***********************/

			 if (stinst.NbChannel > 0)
				{
				 /* Updating Digitizers board numbers */

				 /* SENCA & PAMPCA lines */
				 for (z=1;z<=stinst.NbDigi;z++)
					{
					 char	*ctmp;
					 char	itmp[5];
					 char	otmp[5];


					 sprintf (itmp, "?%d?", DigiBoard[z]);

					 for (zz=1;zz<=stinst.NbSensor;zz++)
						{
						 ctmp = strstr (stinst.Sensor[zz].Senca, itmp);

						 while (ctmp != NULL)
							{
							 sprintf (otmp, "%-2d  ", z);
							 memcpy (ctmp, otmp, 4);

							 ctmp = strstr (stinst.Sensor[zz].Senca, itmp);
							}
						}

					 for (zz=1;zz<=stinst.NbPreAmp;zz++)
						{
						 ctmp = strstr (stinst.PreAmp[zz].Pampca, itmp);

						 while (ctmp != NULL)
							{
							 sprintf (otmp, "%-2d  ", z);
							 memcpy (ctmp, otmp, 4);

							 ctmp = strstr (stinst.PreAmp[zz].Pampca, itmp);
							}
						}
					}

				 /* DIGICA lines */
				 for (x=1;x<=stinst.NbData;x++)
					{
					 char	*ctmp;
					 char	itmp[5];
					 char	otmp[5];


					 for (y=1;y<=stinst.Data[x].NbDigi;y++)
						{
						 for (z=1;z<=stinst.NbDigi;z++)
							{
							 sprintf (itmp, "?%d?", DigiBoard[z]);

							 ctmp = strstr (stinst.Data[x].Digi[y].DIGICA, itmp);

							 while (ctmp != NULL)
								{
								 sprintf (otmp, "%-2d  ", z);
								 memcpy (ctmp, otmp, 4);

								 ctmp = strstr (stinst.Data[x].Digi[y].DIGICA, itmp);
								}
							}
						}
					}

				 /* DLOGCA lines */
				 for (x=1;x<=stinst.NbData;x++)
					{
					 char	*ctmp;
					 char	itmp[5];
					 char	otmp[5];


					 for (z=1;z<=stinst.NbDigi;z++)
						{
						 sprintf (itmp, "?%d?", DigiBoard[z]);

						 ctmp = strstr (stinst.Data[x].Data, itmp);

						 while (ctmp != NULL)
							{
							 sprintf (otmp, "%-2d  ", z);
							 memcpy (ctmp, otmp, 4);

							 ctmp = strstr (stinst.Data[x].Data, itmp);
							}
						}
					}

				 /* Writing STINST line */
				 fprintf (fsta, "\nSTINST\t%d\t%d\t%d\t%d\t%s\t%s\n",
					  stinst.NbSensor, stinst.NbPreAmp, stinst.NbDigi, stinst.NbData,
					  stinst.SDate, stinst.EDate);

				 /* Writing Sensor Information */
				 for (z=1;z<=stinst.NbSensor;z++)
					{
					 /* Writing SENSN line */
					 fprintf (fsta, "SENSN\t%s\t%d\t%s\t%s\t%s\t%s\t%d\n",
							stinst.Sensor[z].SensorSN, z, stinst.Sensor[z].Latitude,
							stinst.Sensor[z].Longitude, stinst.Sensor[z].Elevation,
							stinst.Sensor[z].Depth, stinst.Sensor[z].NbComp+1);

					 /* Writing SENCA lines */
					 fprintf (fsta, "%s", stinst.Sensor[z].Senca);
					}

				 /* Writing Pre-Amplifier Information */
				 for (z=1;z<=stinst.NbPreAmp;z++)
					{
					 /* Writing PAMPSN line */
					 fprintf (fsta, "PAMPSN\t%s\t%d\n",
							stinst.PreAmp[z].PreAmpSN, stinst.PreAmp[z].NbChan+1);

					 /* Writing PAMPCA lines */
					 fprintf (fsta, "%s", stinst.PreAmp[z].Pampca);
					}

				 /* Writing Digitizer Information */
				 for (z=1;z<=stinst.NbData;z++)
					{
					 for (m=1;m<=stinst.Data[z].NbDigi;m++)
						{
						 /* Writing DIGISN line */
						 fprintf (fsta, "DIGISN\t%s\t%d\t%d\n",
							  stinst.Data[z].Digi[m].DigiSN, stinst.Data[z].Digi[m].NbModule, 0);

						 /* Writing DIGICA line */
						 fprintf (fsta, "%s", stinst.Data[z].Digi[m].DIGICA);
						}
					}

				 /* Writing Datalogger Information */
				 for (m=1;m<=stinst.NbData;m++)
					{
					 /* Writing DLOGSN line */
					 fprintf (fsta, "DLOGSN\t%s\t%d\n",
						  stinst.Data[m].DataSN, stinst.Data[m].NbChannel);

					 /* Writing Datalogger Information */
					 fprintf (fsta, "%s", stinst.Data[m].Data);
					}
				}
			}
		}
	 else	printf ("\t\tNo time span.");

	 /* Closing tmp station file */
	 fclose (ftmp);

	 /* Closing tbl station file */
	 fclose (fsta);

	 /* Closing rem station file */
	 fclose (frem);
	}

 /* Closing sensor.tbl file */
 fclose (fsensor);

 /* Closing preamp.tbl file */
 fclose (fpreamp);

 /* Closing datlog.tbl file */
 fclose (fdatlog);

 /* Closing error.tbl file */
 fprintf (ferrortbl, "\n");
 fclose (ferrortbl);

 /* Deleting Temporary & Empty Files */
 for (i=0;i<NbSL;i++)
	{
	 /* Deleting temporary file */
	 sprintf (ntmp, "%s.tmp", SL[i].staname);
	 err = unlink (ntmp);

	 /* Deleting tbl file? */
	 sprintf (ntmp, "%s.tbl", SL[i].staname);
	 if ((stat (ntmp, &buf)) == (-1))
		{
		 printf ("\nError: Couldn't stat file %s.\n", ntmp);

		 exit (-22);
		}
	 else if (buf.st_size == 0)
		{
		 err = unlink (ntmp);
		}
	}

 printf ("\n");

 return (0);
}


/************************************************************************/
/*  Split functions:							*/
/*  The split functions will split a line containing 0 or more tokens	*/
/*  into an argc/argv style list of tokens.				*/
/*  Tokens are SEPARATED by the specified delimiter.			*/
/*  If there are NO characters in the line, there are no tokens.	*/
/*  If there are 2 adjacent delimeters (or adjacent delimiter and	*/
/*  string terminator), that represents a 0 length token.		*/
/************************************************************************/

/************************************************************************/
/*  split_inline:							*/
/*	Split line into tokens based on delimiter.			*/
/*	Push token onto list when start of token is seen.		*/
/*	Do not copy token, so token may be modified later.		*/
/*	Original line is modified.					*/
/************************************************************************/
int split_inline (char *line, char delim)
{
    char *p = line;
    int start_token = 1;
    int force_token = 0;
    for (p=line;;p++) {
	if (*p == 0) {
	    if (force_token) {
		push_token (&tokenc, &tokenv, p, 0);
	    }
	    return (tokenc);
	}
	if (start_token) {
	    push_token (&tokenc, &tokenv, p, 0);
	    start_token = 0;
	    force_token = 0;
	}
	if (*p == delim) {
	    *p = '\0';
	    start_token = 1;
	    force_token = 1;
	}
    }
}

/************************************************************************/
/*  split_copy:								*/
/*	Split line into tokens based on delimiter.			*/
/*	Push token onto list at end of token.				*/
/*	Copy token.  Do not modify original line.			*/
/************************************************************************/
int split_copy (char *line, char delim)
{
    char *p;
    char *s;
    char *str;
    int in_token = 0;
    for (p=line,s=line;;p++) {
	if (*p == 0) {
	    if (in_token) {
		str = strndup(s,p+1-s);
		str[p-s] = '\0';
		push_token (&tokenc, &tokenv, str, 0);
	    }
	    return (tokenc);
	}
	in_token = 1;
	if (*p == delim) {
	    str = strndup(s,p+1-s);
	    str[p-s] = '\0';
	    push_token (&tokenc, &tokenv, str, 0);
	    s = p+1;
	}
    }
}

/************************************************************************/
/*  General token routines.						*/
/************************************************************************/

/************************************************************************/
/*  push_token:								*/
/*	Push a token onto a token list.					*/
/*	Allocate additional space for the expanded token list.		*/
/*	Allocate space for actual token if dup flag is set.		*/
/************************************************************************/
int push_token (int *pargc, char ***pargv, char *p, int dup)
{
    int n = *pargc;
    char **argv = *pargv;
    argv = (n==0) ? (char **)malloc((n+1)*sizeof(char *)) :
		      (char **)realloc(argv,(n+1)*sizeof(char *));
    if (argv == NULL) {
	fprintf (stderr, "Error (re)allocing argv\n");
	exit(1);
    }
    argv[n++] = (dup) ? strdup(p) : p;
    *pargv = argv;
    *pargc = n;
    return (0);
}

/************************************************************************/
/*  reset_token:							*/
/*	Free the space for the tokenv array list.			*/
/*	Reset the token list variables.					*/
/*	Free actual tokens of dup variable is dup is set.		*/
/************************************************************************/
int reset_token (int *pargc, char ***pargv, int dup)
{
    if (dup) {
	int i;
	int argc = *pargc;
	char **argv = *pargv;
	for (i=0; i<argc; i++) {
	    free (argv[i]);
	}
    }
    if (*pargv) free (*pargv);
    *pargv = NULL;
    *pargc = 0;

 return (0);
}

char *strndup (char *p, int n)
{
    char *s, *t;
    s = (char *)malloc(n*sizeof(char));
    if (s == NULL) return (s);
    t = s;
    while (n-->0) *t++ = *p++;
    return s;
}
